SystemOrganization addCategory: #'Ma-Core-Abstractions'!
SystemOrganization addCategory: #'Ma-Core-Constants'!
SystemOrganization addCategory: #'Ma-Core-Domain'!
SystemOrganization addCategory: #'Ma-Core-Domains'!
SystemOrganization addCategory: #'Ma-Core-Exceptions'!
SystemOrganization addCategory: #'Ma-Core-Requests'!
SystemOrganization addCategory: #'Ma-Core-Utilities'!

!SortedCollection methodsFor: '*ma-core' stamp: 'cmm 7/15/2006 15:12'!
maIsSortedCollection
	^ true! !

!Bag methodsFor: '*ma-core' stamp: 'cmm 12/19/2002 21:33'!
maIncludesAll: aCollection
	"Return true if the receiver includes all elements in aCollection.
	If aCollection contains duplicate elements, the receiver must have
	one for each of them."

	| otherColl |

	aCollection size > self size ifTrue: [ ^false ].
	otherColl :=
		aCollection size > 20
			ifTrue: [ aCollection asBag ]
			ifFalse: [ aCollection ].
	otherColl do:
		[ :each | (self occurrencesOf: each) < (otherColl occurrencesOf: each) ifTrue: [ ^false ] ].
	^true! !

!Bag methodsFor: '*ma-core' stamp: 'cmm 11/12/2007 00:01'!
printOn: aStream 
	aStream nextPutAll: 
		(self class name first isVowel 
			ifTrue: [ 'an ' ]
			ifFalse: [ 'a ' ]).
	self species printOn: aStream.
	contents keysAndValuesDo: 
		[ : eachKey : eachValue | 
		aStream
			cr ;
			tab ;
			maPrint: eachValue ;
			space ;
			nextPutAll: 'occurrencesOf: ' ;
			maPrint: eachKey ]! !

SharedPool subclass: #MaCoreConstants
	instanceVariableNames: ''
	classVariableNames: 'HasMagma'
	poolDictionaries: ''
	category: 'Ma-Core-Constants'!

!MaCoreConstants class methodsFor: 'as yet unclassified' stamp: 'cmm 6/22/2012 13:13'!
initialize
	HasMagma := false! !

!DirectoryEntryFile methodsFor: '*ma-core-private' stamp: 'cmm 9/13/2007 17:54'!
contributeStatisticsTo: aDirectoryEntryDirectory 
	"Add my size information to the cumulative total in aDirectoryEntryDirectory."
	aDirectoryEntryDirectory addToTotalSize: self fileSize! !

!DirectoryEntryFile methodsFor: '*ma-core' stamp: 'cmm 2/12/2009 18:44'!
directoryName
	"Answer the directory to shell to for this item."
	^ self containingDirectory pathName! !

!DirectoryEntryFile methodsFor: '*ma-core-private' stamp: 'cmm 9/13/2007 17:49'!
incrementDirectoryOrFileCount: twoElementArray 
	twoElementArray 
		at: 2
		put: (twoElementArray at: 2) + 1! !

!DirectoryEntryFile methodsFor: '*ma-core' stamp: 'cmm 10/11/2009 19:31'!
localFileName
	"For compatibility with ZipArchiveMember."
	^ self name! !

!DirectoryEntryFile methodsFor: '*ma-core' stamp: 'cmm 9/20/2007 17:07'!
totalFileSize
	"API compatible with DirectoryEntryDirectory>>#totalFileSize, but just answers my fileSize."
	^ self fileSize! !

!Set methodsFor: '*ma-core' stamp: 'cmm 7/24/2002 20:41'!
maShallowCopy

	| copy |
	copy := self species new: self size.
	self do: [ :each | copy add: each ].
	^copy! !

!CompiledMethod methodsFor: '*ma-core' stamp: 'cmm 12/17/2008 20:29'!
maSourceString
	^ self getSource asString! !

!Character methodsFor: '*ma-core' stamp: 'cmm 7/30/2007 00:22'!
maPrintOn: aStream
	aStream nextPut: self! !

!Character methodsFor: '*ma-core' stamp: 'cmm 7/24/2002 20:44'!
maShallowCopy
	"Characters are unique."

	^self
! !

!Number methodsFor: '*ma-core' stamp: 'cmm 7/18/2007 21:04'!
maLog: anInteger
	^ self log: anInteger! !

!Number methodsFor: '*ma-core' stamp: 'cm 12/24/2001 16:40'!
maPrintAbbreviatedOn: aStream

	self printOn: aStream
! !

!MessageTally class methodsFor: '*ma-core' stamp: 'cmm 11/10/2010 13:22'!
maSpyOn: spyBlock if: conditionBlock 
	^ conditionBlock
		ifTrue: [ self spyOn: spyBlock ]
		ifFalse: spyBlock! !

!MessageTally class methodsFor: '*ma-core' stamp: 'cmm 11/10/2010 13:41'!
maSpyOn: aBlock toFileNamed: fileName reportOtherProcesses: aBoolean if: conditionBlock 
	conditionBlock value
		ifTrue:
			[ self
				spyOn: aBlock
				toFileNamed: fileName
				reportOtherProcesses: aBoolean ]
		ifFalse: aBlock! !

!ByteArray methodsFor: '*ma-core' stamp: 'cmm 8/13/2009 21:53'!
hostNumber
	"This is here for Pharo compatibility.  It is overridden by Pharo's SocketAddress."
	^ String streamContents: 
		[ : stream | 
		self do: 
			[ : each | 
			stream maPrint: each.
			each = self last ifFalse: [ stream maPrint: $. ] ] ]! !

!ByteArray methodsFor: '*ma-core' stamp: 'cmm 6/8/2008 20:55'!
lzCompressed
	| compressor compressed |
	compressed := RWBinaryOrTextStream on: ByteArray new.
	compressor := GZipWriteStream on: compressed.
	compressor nextPutAll: self.
	compressor close.
	^ compressed contents! !

!ByteArray methodsFor: '*ma-core' stamp: 'cmm 6/8/2008 21:11'!
lzUncompressed
	"I am a ByteArray compressed with lzCompress.  Answer its original, uncompressed, form."
	| compressed |
	compressed := ReadStream on: self.
	^ (GZipReadStream on: compressed) contents! !

!ByteArray methodsFor: '*ma-core' stamp: 'cmm 8/13/2009 21:53'!
maAsIpString
	"Answer the receiver as a dot-delimited ip address String."
	^ self hostNumber! !

!ByteArray methodsFor: '*ma-core' stamp: 'cmm 8/13/2009 22:31'!
maIsLocalHost
	"Host addresses are represented as ByteArray's in Squeak.  Answer whether this host-address represents a localhost address."
	"Efficiently check whether the initial host-address is 127, which is assumed to be the localhost."
	self maAsIpString in: 
		[ : str | 
		(str first = $1 and: [ str second = $2 and: [ str third = $7 ] ]) ifTrue: [ ^ true ] ].
	^ self = NetNameResolver localHostAddress! !

!ByteArray methodsFor: '*ma-core' stamp: 'cmm 8/10/2010 17:49'!
maUint: bits at: anInteger 
	"oids"
	| answer bytes |
	bits == 48 ifTrue: [ ^ self maUnsigned48At: anInteger + 1 ].
	"physicalSize, word buffers"
	bits == 32 ifTrue: 
		[ ^ self 
			unsignedLongAt: anInteger + 1
			bigEndian: false ].
	"control fields - bpMapIndex."
	bits == 8 ifTrue: [ ^ self byteAt: anInteger + 1 ].
	"class ids and classVersions"
	bits == 16 ifTrue: 
		[ ^ self 
			unsignedShortAt: anInteger + 1
			bigEndian: false ].
	bits == 24 ifTrue: [ ^ self maUnsigned24At: anInteger + 1 ].
	bits == 64 ifTrue: [ ^ self maUnsigned64At: anInteger + 1 ].
	bits == 40 ifTrue: [ ^ self maUnsigned40At: anInteger + 1 ].
	bits == 56 ifTrue: [ ^ self maUnsigned56At: anInteger + 1 ].
	bytes := bits // 8.
	answer := LargePositiveInteger new: bytes.
	1 
		to: bytes
		do: 
			[ : digitPosition | 
			answer 
				digitAt: digitPosition
				put: (self at: digitPosition + anInteger) ].
	^ answer normalize! !

!ByteArray methodsFor: '*ma-core' stamp: 'cmm 3/18/2005 10:30'!
maUint: bits at: position put: anInteger

	position + 1
		to: position + (bits // 8)
		do:
			[ :pos |
			self
				at: pos
				put: (anInteger digitAt: pos-position) ].
	^anInteger! !

!ByteArray methodsFor: '*ma-core' stamp: 'cmm 12/19/2008 14:30'!
maUnsigned24At: index
	"Return a 40-bit unsigned integer quantity starting from the given byte index"
	| b0 b1 w |
	w := self at: index.
	b1 := self at: index+1.
	b0 := self at: index+2.
	b1 = 0 ifFalse:[w := (b1 bitShift: 8) + w].
	b0 = 0 ifFalse:[w := (b0 bitShift: 16) + w].
	^w! !

!ByteArray methodsFor: '*ma-core' stamp: 'cmm 12/12/2008 16:09'!
maUnsigned40At: index
	"Return a 40-bit unsigned integer quantity starting from the given byte index"

	| b0 b1 b2 b3 w |

	w := self at: index.
	b3 := self at: index+1.
	b2 := self at: index+2.
	b1 := self at: index+3.
	b0 := self at: index+4.

	"Minimize LargeInteger arithmetic"
	b3 = 0 ifFalse:[w := (b3 bitShift: 8) + w].
	b2 = 0 ifFalse:[w := (b2 bitShift: 16) + w].
	b1 = 0 ifFalse:[w := (b1 bitShift: 24) + w].
	b0 = 0 ifFalse:[w := (b0 bitShift: 32) + w].

	^w! !

!ByteArray methodsFor: '*ma-core' stamp: 'cmm 12/12/2008 16:09'!
maUnsigned48At: index
	"Return a 48-bit unsigned integer quantity starting from the given byte index"

	| b0 b1 b2 b3 b4 w |

	w := self at: index.
	b4 := self at: index+1.
	b3 := self at: index+2.
	b2 := self at: index+3.
	b1 := self at: index+4.
	b0 := self at: index+5.

	"Minimize LargeInteger arithmetic"
	b4 = 0 ifFalse:[w := (b4 bitShift: 8) + w].
	b3 = 0 ifFalse:[w := (b3 bitShift: 16) + w].
	b2 = 0 ifFalse:[w := (b2 bitShift: 24) + w].
	b1 = 0 ifFalse:[w := (b1 bitShift: 32) + w].
	b0 = 0 ifFalse:[w := (b0 bitShift: 40) + w].

	^w! !

!ByteArray methodsFor: '*ma-core' stamp: 'cmm 12/12/2008 16:08'!
maUnsigned56At: index
	"Return a 56-bit unsigned integer quantity starting from the given byte index"

	| b0 b1 b2 b3 b4 b5 w |

	w := self at: index.
	b5 := self at: index+1.
	b4 := self at: index+2.
	b3 := self at: index+3.
	b2 := self at: index+4.
	b1 := self at: index+5.
	b0 := self at: index+6.

	"Minimize LargeInteger arithmetic"
	b5 = 0 ifFalse:[w := (b5 bitShift: 8) + w].
	b4 = 0 ifFalse:[w := (b4 bitShift: 16) + w].
	b3 = 0 ifFalse:[w := (b3 bitShift: 24) + w].
	b2 = 0 ifFalse:[w := (b2 bitShift: 32) + w].
	b1 = 0 ifFalse:[w := (b1 bitShift: 40) + w].
	b0 = 0 ifFalse:[w := (b0 bitShift: 48) + w].

	^w! !

!ByteArray methodsFor: '*ma-core' stamp: 'cmm 3/18/2005 09:59'!
maUnsigned64At: index
	"Return a 32bit unsigned integer quantity starting from the given byte index"

	| b0 b1 b2 b3 b4 b5 b6 w |

	w := self at: index.
	b6 := self at: index+1.
	b5 := self at: index+2.
	b4 := self at: index+3.
	b3 := self at: index+4.
	b2 := self at: index+5.
	b1 := self at: index+6.
	b0 := self at: index+7.

	"Minimize LargeInteger arithmetic"
	b6 = 0 ifFalse:[w := (b6 bitShift: 8) + w].
	b5 = 0 ifFalse:[w := (b5 bitShift: 16) + w].
	b4 = 0 ifFalse:[w := (b4 bitShift: 24) + w].
	b3 = 0 ifFalse:[w := (b3 bitShift: 32) + w].
	b2 = 0 ifFalse:[w := (b2 bitShift: 40) + w].
	b1 = 0 ifFalse:[w := (b1 bitShift: 48) + w].
	b0 = 0 ifFalse:[w := (b0 bitShift: 56) + w].

	^w! !

!Delay methodsFor: '*ma-core' stamp: 'cmm 9/1/2004 22:25'!
maReset
	"If I am already waiting, start my wait back at the beginning.  If I'm not yet waiting, no effect."
	resumptionTime := Time millisecondClockValue + delayDuration! !

Error subclass: #MaError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Exceptions'!

!MaError commentStamp: '<historical>' prior: 0!
Base abstraction for errors that occur in Ma code.!

MaError subclass: #MaCorruptionError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Exceptions'!

MaError subclass: #MaEnvironmentError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Exceptions'!

MaError subclass: #MaSoftwareError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Exceptions'!

!MaSoftwareError commentStamp: 'cmm 5/4/2005 10:25' prior: 0!
A software error is a bug in the software.  Signal them when you have to code for something that "should never happen."  For example, if, based on your program logic, there should be an entry in a Dictionary, but it is not there, you would signal a software error.!

MaError subclass: #MaUserError
	instanceVariableNames: 'page'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Exceptions'!

!MaUserError commentStamp: '<historical>' prior: 0!
I represent an error made by the user.!

!MaUserError class methodsFor: 'instance creation' stamp: 'cmm 2/6/2008 16:22'!
page: aMauiPage 
	^ self new
		page: aMauiPage ;
		yourself! !

!MaUserError class methodsFor: 'instance creation' stamp: 'cmm 2/6/2008 16:22'!
signalPage: aMauiPage 
	(self page: aMauiPage) signal! !

!MaUserError methodsFor: 'testing' stamp: 'cmm 9/28/2008 18:18'!
isMaUserError

	^true! !

!MaUserError methodsFor: 'accessing' stamp: 'cmm 2/6/2008 16:21'!
page
	^ page ifNil: [ MauiPage text: self messageText ]! !

!MaUserError methodsFor: 'accessing' stamp: 'cmm 7/12/2010 23:36'!
page: aMauiPage 
	page := aMauiPage.
	self messageText: aMauiPage string! !

MaUserError subclass: #MauiUserError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Exceptions'!

!MauiUserError methodsFor: 'accessing' stamp: 'cmm 6/5/2012 22:03'!
mauiName
	^ 'Error'! !

!WriteStream methodsFor: '*ma-core' stamp: 'cmm 11/10/2002 20:35'!
endEntry
	"No-op for compatibility with TranscriptStream."! !

!WriteStream methodsFor: '*ma-core' stamp: 'cmm 7/30/2007 00:23'!
maPrint: anObject 
	anObject maPrintOn: self! !

!WriteStream methodsFor: '*ma-core' stamp: 'cm 12/24/2001 16:03'!
maPrintAbbreviated: aMaObject

	aMaObject maPrintAbbreviatedOn: self
! !

!SecureHashAlgorithm methodsFor: '*ma-core' stamp: 'cmm 6/9/2005 11:55'!
maHashMessage: aStringOrByteArray from: startPos to: stopPos
	"Hash the given message using the Secure Hash Algorithm."

	^ self
		maHashStream: 
			((ReadStream on: aStringOrByteArray asByteArray)
				position: startPos-1 ;
				yourself)
		to: stopPos! !

!SecureHashAlgorithm methodsFor: '*ma-core' stamp: 'cmm 6/9/2005 11:59'!
maHashStream: aPositionableStream to: endPosition
	"Hash the contents of the given stream from the current position to endPosition using the Secure Hash Algorithm. The SHA algorithm is defined in FIPS PUB 180-1. It is also described on p. 442 of 'Applied Cryptography: Protocols, Algorithms, and Source Code in C' by Bruce Scheier, Wiley, 1996."

	| currentPosition buf bitLength |
	self initializeTotals.
	aPositionableStream atEnd ifTrue: [self error: 'empty stream'].
	endPosition > aPositionableStream size ifTrue: [ self error: 'invalid endPosition' ].
	[ currentPosition := aPositionableStream position.
	currentPosition < endPosition ]
		whileTrue:
			[ buf := aPositionableStream next: (64 min: endPosition-currentPosition).
			(currentPosition < endPosition
			and: [buf size = 64])
				ifTrue: [ self processBuffer: buf ]
				ifFalse: 
					[ bitLength := (endPosition - currentPosition) * 8.
					self processFinalBuffer: buf bitLength: bitLength ] ].
	^ self finalHash! !

!Behavior methodsFor: '*ma-core' stamp: 'cmm 12/2/2004 15:07'!
maAllInstVarNamesDo: aBlock

	superclass ifNotNil: [ superclass maAllInstVarNamesDo: aBlock ].
	self instVarNames do: aBlock! !

!Behavior methodsFor: '*ma-core' stamp: 'cmm 12/16/2004 01:26'!
maAllSelectorsDo: aBlock

	self selectorsDo: aBlock.
	superclass ifNotNil: [ superclass maAllSelectorsDo: aBlock ]! !

!Behavior methodsFor: '*ma-core' stamp: 'cmm 12/3/2007 21:13'!
maFirstCommentAt: selector 
	"A much more-convenient method than firstCommentAt:, which requires you to specify the exact class, a problem when the method is implemented in the superclass methodDict."
	^ (self methodDict includesKey: selector) 
		ifTrue: [ self firstCommentAt: selector ]
		ifFalse: 
			[ self superclass 
				ifNil: [ '' ]
				ifNotNil: [ self superclass maFirstCommentAt: selector ] ]! !

!Behavior methodsFor: '*ma-core' stamp: 'cmm 11/19/2003 22:53'!
maFirstCommentOrNilAt: selector
	"A much more-convenient method than firstCommentAt:, which requires you to specify the exact class, a problem when the method is implemented in the superclass methodDict."

	| string |
	^(string := self maFirstCommentAt: selector) isEmpty
		ifTrue: [ nil ]
		ifFalse: [ string ]! !

!Behavior methodsFor: '*ma-core' stamp: 'cmm 3/8/2011 19:59'!
maPackageInfo
	^ PackageInfo allPackages 
		detect: [ : each | each includesClass: self ]
		ifFound: [ : foundPackage | PackageInfo named: foundPackage packageName ]
		ifNone: [ nil ]! !

!SmalltalkImage methodsFor: '*ma-core' stamp: 'cmm 7/17/2010 14:43'!
defaultMaxMemory
	"Images can be 1GB, however answer a lower number, to give more than ample room for emergency measures to have room to operate (e.g., to commit to a Magma repository)"
	^ 550000000! !

!SmalltalkImage methodsFor: '*ma-core' stamp: 'cmm 10/10/2010 21:30'!
maAuthorInitials
	^ (Smalltalk hasClassNamed: #Utilities)
		ifTrue:
			[ | init |
			init := (Smalltalk classNamed: #Utilities) authorInitialsPerSe.
			init isEmptyOrNil
				ifTrue: [ 'unknown' ]
				ifFalse: [ init ] ]
		ifFalse: [ 'unknown' ]! !

!SmalltalkImage methodsFor: '*ma-core' stamp: 'cmm 7/26/2010 11:32'!
maIsHeadless
	"Display global appears to be set even when headless.  So checking the -vm arguments looking for headless operation."
	-1000 
		to: -1
		do: 
			[ : n | 
			(#('display=none' '-headless' '-vm-display-null') includes: (self getSystemAttribute: n)) ifTrue: [ ^ true ] ].
	^ false! !

!SmalltalkImage methodsFor: '*ma-core' stamp: 'cmm 9/23/2008 14:59'!
maIsLowOnMemory
	"Answer whether the VM is within ten-million bytes of available memory.  The goal is to avoid the LowSpace warning."
	^ self maMemoryAvailable < 10000000! !

!SmalltalkImage methodsFor: '*ma-core' stamp: 'cmm 9/23/2008 15:54'!
maMaxMemory
	"If the special argument to the vm, -memory, was provided, use number following it.  Else 450MB."
	| foundMemoryArg |
	foundMemoryArg := false.
	self maVmParametersDo: 
		[ : each | 
		'-memory' = each 
			ifTrue: [ foundMemoryArg := true ]
			ifFalse: 
				[ foundMemoryArg ifTrue: 
					[ | arg |
					arg := each.
					(arg isEmptyOrNil or: [ arg size < 2 ]) ifFalse: 
						[ | requestedMemory |
						requestedMemory := (arg 
							copyFrom: 1
							to: arg size - 1) asNumber.
						requestedMemory > 0 ifTrue: 
							[ arg last = $k 
								ifTrue: [ ^ 1000 * requestedMemory ]
								ifFalse: [ arg last = $m ifTrue: [ ^ 1000000 * requestedMemory ] ] ] ] ] ] ].
	^ self defaultMaxMemory! !

!SmalltalkImage methodsFor: '*ma-core' stamp: 'cmm 9/23/2008 15:06'!
maMemoryAvailable
	"Answer the remaining memory available accessible by this image."
	^ self maMaxMemory - self maMemoryUsed! !

!SmalltalkImage methodsFor: '*ma-core' stamp: 'cmm 9/23/2008 15:59'!
maMemoryUsed
	^ (self vmParameterAt: 2) max: (self vmParameterAt: 1)! !

!SmalltalkImage methodsFor: '*ma-core' stamp: 'cmm 4/11/2010 18:09'!
maSaveAs: newName 
	"Save the image  under that new name."
	newName ifNil: [ ^ self ].
	(SourceFiles at: 2) ifNotNil: 
		[ self
			closeSourceFiles ;
			saveChangesInFileNamed: (self fullNameForChangesNamed: newName)	"so copying the changes file will always work" ].
	self saveImageInFileNamed: (self fullNameForImageNamed: newName)! !

!SmalltalkImage methodsFor: '*ma-core' stamp: 'cmm 9/23/2008 15:49'!
maVmParameters
	| answer |
	answer := OrderedCollection new.
	self maVmParametersDo: [ : each | answer add: each ].
	^ answer! !

!SmalltalkImage methodsFor: '*ma-core' stamp: 'cmm 9/23/2008 15:51'!
maVmParametersDo: aBlock 
	"Display global appears to be set even when headless.  So checking the -vm arguments looking for headless operation."
	-1 
		to: -1000
		by: -1
		do: 
			[ : n | 
			| attr |
			attr := self getSystemAttribute: n.
			attr ifNotNil: [ aBlock value: attr ] ]! !

!UndefinedObject methodsFor: '*ma-core' stamp: 'cmm 10/31/2009 12:31'!
hand
	^ ActiveHand! !

!UndefinedObject methodsFor: '*ma-core' stamp: 'cmm 5/23/2012 21:37'!
hash
	"Squeak's hash for nil since 2009.  Allow this literal to have its own value hash independent of its identityHash, to be safer with distributed systems which may be using nil in a hash calculation."
	^ 1006632960! !

!String methodsFor: '*ma-core' stamp: 'cmm 7/6/2006 12:11'!
between: minString and: maxString
	^ self >= minString  and: [self <= maxString ]! !

!String methodsFor: '*ma-core' stamp: ''!
maAsIntegerFromRadix: anInteger
	"Answer a Smalltalk Integer with an equivalent value of the receiver
	represented in radix anInteger."

	^self
		inject: 0
		into: [ :sum :character | (sum * anInteger) + character digitValue ]
! !

!String methodsFor: '*ma-core' stamp: 'cmm 8/6/2008 21:19'!
maAsNumber
	"Squeak..  Depending on *how* the receiver is not numeric, It sometimes answers 0, other times signals an error."
	| n |
	^ 
	[ n := self asNumber.
	n = 0 ifFalse: [ n ] ] 
		on: Error
		do: [ : err | err return: nil ]! !

!String methodsFor: '*ma-core' stamp: 'cmm 2/27/2008 23:50'!
maBeginsWith: aString caseSensitive: aBoolean 
	"Returns true if the receiver begins with the elements in aString, while ignoring the case if aBoolean is true."
	self size < aString size ifTrue: [ ^ false ].
	"Optimization.  Most strings will differ on the first character.."
	self first asLowercase = aString first asLowercase ifFalse: [ ^ false ].
	^ (self 
		findSubstring: aString
		in: self
		startingAt: 1
		matchTable: (aBoolean 
				ifTrue: [ CaseSensitiveOrder ]
				ifFalse: [ CaseInsensitiveOrder ])) = 1! !

!String methodsFor: '*ma-core' stamp: 'cmm 2/28/2008 00:14'!
maEndsWith: aString caseSensitive: aBoolean 
	"Returns true if the receiver begins with the elements in aString,while ignoring the case if aBoolean is true."
	| extra |
	(extra := self size - aString size) < 0 ifTrue: [ ^ false ].
	^ (self 
		findSubstring: aString
		in: self
		startingAt: extra + 1
		matchTable: (aBoolean 
				ifTrue: [ CaseSensitiveOrder ]
				ifFalse: [ CaseInsensitiveOrder ])) > 0! !

!String methodsFor: '*ma-core' stamp: 'cmm 2/7/2008 17:04'!
maFormattedForPlural: aNumber 
	"This is kind of a wing-nut method, but it can save you some
	redundant code if you care to make your programs user message
	plural and singular friendly.  It will add or remove an s based on
	aNumber and whether or not the receiver already ends in s."
	^ aNumber > 0 & (aNumber <= 1) 
		ifTrue: 
			[ "should not end in s"
			(self 
				maEndsWith: 's'
				caseSensitive: true) 
				ifTrue: 
					[ self 
						copyFrom: 1
						to: self size - 1 ]
				ifFalse: [ self ] ]
		ifFalse: 
			[ (self 
				maEndsWith: 's'
				caseSensitive: true) 
				ifTrue: 
					[ "should end in s"
					self ]
				ifFalse: [ self , 's' ] ]! !

!String methodsFor: '*ma-core' stamp: 'cmm 7/30/2007 00:18'!
maPrintAbbreviatedOn: aStream 
	aStream nextPutAll: self! !

!String methodsFor: '*ma-core' stamp: 'cmm 7/30/2007 00:22'!
maPrintOn: aStream
	aStream nextPutAll: self! !

!String methodsFor: '*ma-core' stamp: 'cmm 2/2/2008 16:55'!
occurrence: anInteger where: oneArgBlock 
	^ self 
		occurrence: anInteger
		where: oneArgBlock
		startingAt: 1! !

!String methodsFor: '*ma-core' stamp: 'cmm 2/2/2008 17:04'!
occurrence: anInteger where: oneArgBlock startingAt: startPos 
	"Answer the position of the anInteger'th Character, aCharacter, in the receiver.  If not found, answer 0."
	| pos count |
	count := 0.
	pos := startPos.
	[ pos < self size and: [ count < anInteger ] ] whileTrue: 
		[ (oneArgBlock value: (self at: pos)) ifTrue: [ count := count + 1 ].
		pos := pos + 1 ].
	^ count = anInteger 
		ifTrue: [ pos-1 ]
		ifFalse: [ 0 ]! !

!String methodsFor: '*ma-core' stamp: 'cmm 11/5/2007 01:00'!
smalltalkToWords
	"smalltalkString may be a Smalltalk class name, method name, or a parameter name (aString or smalltalkString).
	
	'MauiUserStrategy' answers 'Maui User Strategy'
	'smalltalkToWords:' answers 'Smalltalk To Words:'
	'aString' answers 'A String'
	'aMaObjectSerializer' answers 'A Ma Object Serializer'
	'aMagmaSession' answers 'A Magma Session'
	'A typical overridden name:' answers 'A typical overridden name:'"
	| smalltalkStream words |
	smalltalkStream := self readStream.
	words := String streamContents: 
		[ : stream | 
		[ smalltalkStream atEnd ] whileFalse: 
			[ (smalltalkStream peek isUppercase and: [ stream isEmpty not and: [ stream last ~= Character space ] ]) ifTrue: [ stream space ].
			stream isEmpty 
				ifTrue: [ stream nextPut: smalltalkStream next asUppercase ]
				ifFalse: [ stream nextPut: smalltalkStream next ] ] ].
	^ words! !

!PositionableStream methodsFor: '*ma-core' stamp: 'cmm 4/2/2009 11:48'!
maReset
	"Reset my position back to the beginning."
	self reset.
	readLimit := collection size! !

!SystemNavigation class methodsFor: '*ma-core' stamp: 'cmm 3/3/2004 22:57'!
maDefault: aSystemNavigation

	Default := aSystemNavigation! !

!SystemNavigation methodsFor: '*ma-core' stamp: 'cmm 1/2/2005 23:31'!
maAllMethodReferencesDo: aBlock
	"Value aBlock for every method in the system."
	Cursor execute showWhile:
		[ self
			allBehaviorsDo:
				[ : class |
				class selectorsDo:
					[ : sel | 
					aBlock value: (MethodReference class: class selector: sel) ] ] ]! !

!Integer methodsFor: '*ma-core' stamp: 'cmm 11/19/2009 20:56'!
billion
	^self*1000000000! !

!Integer methodsFor: '*ma-core' stamp: 'cmm 11/19/2009 20:56'!
hundred
	^self*100! !

!Integer methodsFor: '*ma-core' stamp: 'cmm 3/22/2011 20:33'!
maBitClear: index 
	"Return the receiver with the bit at index cleared."
	^ self
		bitAt: index
		put: 0! !

!Integer methodsFor: '*ma-core' stamp: 'cmm 3/22/2011 20:33'!
maBitSet: index 
	"Return the receiver with the bit at index set."
	^ self
		bitAt: index
		put: 1! !

!Integer methodsFor: '*ma-core' stamp: 'cmm 9/1/2006 17:51'!
maIsBitClear: bitPosition 
	^ (self maIsBitSet: bitPosition) not! !

!Integer methodsFor: '*ma-core' stamp: 'cmm 9/1/2006 17:52'!
maIsBitSet: bitPosition 
	^ self 
		anyBitOfMagnitudeFrom: bitPosition
		to: bitPosition! !

!Integer methodsFor: '*ma-core' stamp: 'cmm 11/19/2009 20:56'!
million
	^self*1000000! !

!Integer methodsFor: '*ma-core' stamp: 'cmm 11/19/2009 20:56'!
thousand
	^self*1000! !

!BlockClosure methodsFor: '*ma-core' stamp: 'cmm 8/13/2009 21:33'!
maOn: exc1 do: block1 on: exc2 do: block2

	^[
		[ self value ]
			on: exc1
			do: block1 ]

		on: exc2
		do: block2! !

!BlockClosure methodsFor: '*ma-core' stamp: 'cmm 8/13/2009 21:33'!
maOn: exc1 do: block1 on: exc2 do: block2 on: exc3 do: block3

	^[
		[ self value ]
			on: exc1
			do: block1 ]

		maOn: exc2
		do: block2

		on: exc3
		do: block3! !

!BlockClosure methodsFor: '*ma-core' stamp: 'cmm 11/20/2012 14:59'!
referencesDo: aBlock
	aBlock value: self receiver! !

!BlockClosure methodsFor: '*ma-core' stamp: 'cmm 11/19/2010 10:29'!
sentTo: anObject
	^ self value: anObject! !

!BlockClosure methodsFor: '*ma-core' stamp: 'cmm 10/22/2012 14:31'!
tempsDo: aBlock 
	1
		to: self outerContext numTemps
		do:
			[ : slot | aBlock value: (self outerContext tempAt: slot) ]! !

!BlockClosure methodsFor: '*ma-core' stamp: 'cmm 10/7/2010 20:35'!
valueWithAllArguments: anArray 
	"Same as valueWithPossibleArgs: except for a one-arg block, which is sure to pass all of anArray rather than just the first of its arguments.  This is used to allow a large, variable number of arguments to be passed."
	^numArgs = 0
		ifTrue: [self value]
		ifFalse:
			[self valueWithPossibleArgs:
				(numArgs = anArray size
					ifTrue: [anArray]
					ifFalse:
						[numArgs > anArray size
							ifTrue: [anArray, (Array new: numArgs - anArray size)]
							ifFalse:
								[numArgs = 1
									ifTrue: [Array with: anArray]
									ifFalse: [anArray copyFrom: 1 to: numArgs]]])]! !

!OrderedCollection methodsFor: '*ma-core' stamp: 'cm 1/7/2002 16:25'!
maAddAll: aSequenceableCollection afterIndex: integerIndex

	aSequenceableCollection reverseDo: [:each | self add: each afterIndex: integerIndex ].
	^aSequenceableCollection
! !

!Interval methodsFor: '*ma-core' stamp: 'cmm 6/4/2005 17:22'!
maEnvelopes: anInterval 
	"Return true if anInterval fits entirely inside the receiver."

	^ (anInterval first between: start and: stop)
	and: [ anInterval last between: start and: stop ]! !

!Interval methodsFor: '*ma-core' stamp: 'cmm 6/4/2005 01:09'!
maIntersects: anInterval 
	"Return true if the receiver overlaps with anInterval."

	^(start between: anInterval first and: anInterval last) 
		or: [ anInterval first between: start and: stop ]! !

!Collection methodsFor: '*ma-core' stamp: 'cmm 7/9/2002 22:59'!
maConsistsOf: aCollection
	"Answer whether the receiver and aCollection reference all equal
	objects, including the same number of references to those objects.  Order
	is not a factor."

	^self size = aCollection size and: [ self maIncludesAll: aCollection ]
! !

!Collection methodsFor: '*ma-core' stamp: 'cmm 1/16/2002 16:48'!
maDetect: aBlock ifFound: foundBlock ifNone: exceptionBlock 
	"foundBlock takes one argument, the found object."

	self 
		do: [ :element | (aBlock value: element) ifTrue: [ ^foundBlock value: element ] ].
	^exceptionBlock value
! !

!Collection methodsFor: '*ma-core' stamp: 'cmm 7/24/2004 01:05'!
maDo: aBlock while: conditionBlock

	conditionBlock value ifFalse: [ ^ self ].
	self do:
		[ : each |
		aBlock value: each.
		conditionBlock value ifFalse: [ ^ self ] ]! !

!Collection methodsFor: '*ma-core' stamp: 'cmm 12/19/2002 21:32'!
maIncludesAll: aCollection
	"Return true if the receiver includes all elements in aCollection.
	If aCollection contains duplicate elements, the receiver must have
	one for each of them."

	| otherColl |

	aCollection size > self size ifTrue: [ ^false ].
	self size > 20
		ifTrue:
			[ ^self asBag maIncludesAll: aCollection ].
	otherColl :=
		aCollection size > 20
			ifTrue: [ aCollection asBag ]
			ifFalse: [ aCollection ].
	otherColl do:
		[ :each | (self occurrencesOf: each) < (otherColl occurrencesOf: each) ifTrue: [ ^false ] ].
	^true! !

!Collection methodsFor: '*ma-core' stamp: ''!
maMatchesElementsOf: anotherCollection
	"Answer whether the receiver points to equivalent objects as
	anotherCollection.  In SequencableCollections, the order must be
	the same as well."

	^self maConsistsOf: anotherCollection
! !

!Collection methodsFor: '*ma-core' stamp: ''!
maRemoveAll: aCollection forEachAbsent: aBlock

	^aCollection do:
		[ :element |
		self
			remove: element
			ifAbsent: [ aBlock value: element ] ]
! !

!DirectoryEntry methodsFor: '*ma-core' stamp: 'cmm 2/12/2009 18:56'!
directoryName
	"Answer the directory to shell to for this item."
	self subclassResponsibility ! !

!DirectoryEntry methodsFor: '*ma-core' stamp: 'cmm 11/27/2011 23:09'!
maFileSizeString
	"Answer my file size as an easy-to-read String."
	^ self fileSize asBytesDescription! !

!DirectoryEntry methodsFor: '*ma-core' stamp: 'cmm 11/27/2011 23:10'!
maTotalFileSizeString
	"Answer my file size as an easy-to-read String."
	^ self totalFileSize asBytesDescription! !

!ReadStream methodsFor: '*ma-core' stamp: 'cmm 7/1/2003 22:12'!
maUpToIncluding: anObject

	| start end |
	start := position+1.
	end := collection indexOf: anObject startingAt: start ifAbsent: [ 0 ].

	"not present--return rest of the collection"	
	end = 0 ifTrue: [ ^self upToEnd ].

	"skip to the end and return the data passed over"
	position := end.
	^collection copyFrom: start to: end! !

!SolidFillStyle methodsFor: '*ma-core' stamp: 'cmm 12/8/2008 13:38'!
addFillStyleMenuItems: aMenu hand: aHand from: aMorph 
	"Add the items for changing the current fill style of the receiver"
	aMenu 
		add: 'change color...' translated
		target: self
		selector: #changeColorIn:event:
		argument: aMorph! !

!SolidFillStyle methodsFor: '*ma-core' stamp: 'cmm 12/8/2008 13:41'!
changeColorIn: aMorph event: evt
	"Note: This is just a workaround to make sure we don't use the old color inst var"
	aMorph changeColorTarget: aMorph selector: #fillStyle: originalColor: self hand: evt hand! !

!Timespan methodsFor: '*ma-core' stamp: 'cmm 8/3/2010 22:58'!
maPrintAbbreviatedOn: aStream
	self printOn: aStream! !

!SequenceableCollection methodsFor: '*ma-core' stamp: 'cmm 10/28/2011 15:53'!
at: anObject ifPresent: oneArgBlock ifAbsent: zeroArgBlock 
	| current uniqueObj |
	uniqueObj := String empty.
	^ (current := self 
		at: anObject
		ifAbsent: [ uniqueObj ]) == uniqueObj 
		ifTrue: [ zeroArgBlock value ]
		ifFalse: [ oneArgBlock value: current ]! !

!SequenceableCollection methodsFor: '*ma-core' stamp: 'cmm 5/18/2004 12:01'!
maMatchesElementsOf: anotherCollection
	"Answer whether the receiver points to equivalent objects as
	anotherCollection and in the same order."

	self == anotherCollection ifTrue: [ ^ true ].
	self size = anotherCollection size ifFalse: [ ^false ].
	self
		with: anotherCollection
		do: [ :eachInSelf :eachInOther | eachInSelf = eachInOther ifFalse: [ ^false ] ].
	^true
! !

!SequenceableCollection methodsFor: '*ma-core' stamp: 'cmm 1/20/2010 09:24'!
maPermutations
	^ Array streamContents: 
		[ : stream | 
		self maPermutationsDo: [ : eachCombo | stream nextPut: eachCombo copy ] ]! !

!SequenceableCollection methodsFor: '*ma-core' stamp: 'cmm 1/16/2010 18:49'!
maPermutationsDo: aBlock 
	"I am a collection of other SequenceableCollections.  For each combination of elements in my element-Collections, value aBlock with the arguments Array of each combination of elements."
	| numberOfCombinations args valueAtBlock permutationsSpec |
	permutationsSpec := self select: [ : each | each notEmpty ].
	permutationsSpec ifEmpty: [ ^ self ].
	numberOfCombinations := permutationsSpec 
		inject: 1
		into: [ : prod : each | each size * prod ].
	args := Array new: permutationsSpec size.
	"Eliminate empty Array specifications, rather than complicate enumeration logic, below."
	valueAtBlock := 
	[ : elementIndex | 
	1 
		to: permutationsSpec size
		do: 
			[ : innerIndex | 
			| eachInnerArray radix |
			eachInnerArray := permutationsSpec at: innerIndex.
			radix := 1.
			1 
				to: innerIndex - 1
				do: [ : eachOrder | radix := radix * (permutationsSpec at: eachOrder) size ].
			args 
				at: innerIndex
				put: (eachInnerArray at: elementIndex // radix \\ eachInnerArray size + 1) ].
	aBlock value: args ].
	"With this formula, the last is the first, the first is the second."
	valueAtBlock value: numberOfCombinations.
	1 
		to: numberOfCombinations - 1
		do: valueAtBlock! !

!SequenceableCollection methodsFor: '*ma-core' stamp: 'cmm 10/7/2010 20:26'!
maPermutationsDo: aBlock separatedBy: separatorBlock 
	"I am a collection of other SequenceableCollections.  For each combination of elements in my element-Collections, value aBlock with the arguments Array of each combination of elements, and evaluate the separatorBlock between."
	| beforeFirst |
	beforeFirst := true.
	self maPermutationsDo:
		[ : each | beforeFirst
			ifTrue: [ beforeFirst := false ]
			ifFalse: [ separatorBlock value ].
		aBlock value: each ]! !

!SequenceableCollection methodsFor: '*ma-core' stamp: 'cmm 11/2/2010 15:09'!
maPermutationsSize
	"Assume I am an Array of Array's that will be permuted via maPermuationsDo:.  Progress-indication is useful in this situation."
	^ self
		inject: 1
		into:
			[ : prod : each | prod * (each size max: 1) ]! !

!SequenceableCollection methodsFor: '*ma-core' stamp: 'cmm 7/24/2011 16:46'!
maPermutationsWithIndexesDo: aBlock 
	"I am a collection of other SequenceableCollections.  For each combination of elements in my element-Collections, value aBlock with the arguments Array of each combination of elements."
	| numberOfCombinations args valueAtBlock permutationsSpec indexes |
	"Eliminate empty Array specifications, rather than complicate enumeration logic."
	permutationsSpec := self select: [ : each | each notEmpty ].
	permutationsSpec ifEmpty: [ ^ self ].
	numberOfCombinations := permutationsSpec maPermutationsSize.
	args := Array new: permutationsSpec size.
	indexes := Array new: permutationsSpec size.
	valueAtBlock := 
	[ : elementIndex | 
	1 
		to: permutationsSpec size
		do: 
			[ : innerIndex | | eachInnerArray innerArrayIndex radix |
			eachInnerArray := permutationsSpec at: innerIndex.
			radix := 1.
			1 
				to: innerIndex - 1
				do: [ : eachOrder | radix := radix * (permutationsSpec at: eachOrder) size ].
			innerArrayIndex := elementIndex // radix \\ eachInnerArray size + 1.
			indexes at: innerIndex put: innerArrayIndex.
			args 
				at: innerIndex
				put: (eachInnerArray at: innerArrayIndex) ].
	aBlock value: args value: indexes ].
	"With this formula, the last is the first, the first is the second."
	valueAtBlock value: numberOfCombinations.
	1 
		to: numberOfCombinations - 1
		do: valueAtBlock! !

!MCDirectoryRepository methodsFor: '*ma-core' stamp: 'cmm 3/18/2009 21:09'!
isDirectoryRepository
	^ true! !

!Rectangle class methodsFor: '*ma-core' stamp: 'cmm 9/30/2009 14:08'!
maFullyInverted
	^ self 
		origin: Float infinity @ Float infinity
		corner: Float infinity negated @ Float infinity negated! !

!Rectangle methodsFor: '*ma-core' stamp: 'cmm 10/30/2009 22:57'!
isInverted
	^ origin x > corner x! !

!Rectangle methodsFor: '*ma-core' stamp: 'cmm 4/9/2010 12:55'!
maCloseTo: aRectangle 
	^ (aRectangle topLeft maCloseTo: self topLeft) and: [ aRectangle bottomRight maCloseTo: self bottomRight ]! !

!Rectangle methodsFor: '*ma-core' stamp: 'cmm 6/5/2010 22:27'!
maLongestSide
	^ self width max: self height! !

!Rectangle methodsFor: '*ma-core' stamp: 'cmm 6/23/2010 22:52'!
maPointAtRandom
	^(self topLeft x + (self width * Collection randomForPicking next)) @ (self topLeft y + (self height * Collection randomForPicking next))! !

!Rectangle methodsFor: '*ma-core' stamp: 'cmm 3/27/2010 15:37'!
maPreciseCenter
	"Same as #center, but usable for Rectangles specified by Floating points, in that it does not truncate to an Integer as #center does."
	^self topLeft + self bottomRight / 2! !

!FileDirectory class methodsFor: '*ma-core' stamp: 'cmm 4/21/2009 22:12'!
maDirectoryEntryFor: fullyQualifiedFilename 
	"Given fullyQualifiedFilename, answer a DirectoryEntryFile if fullyQualifiedFilename is a file, or a DirectoryEntryDirectory if it refers to a directory."
	^ (FileDirectory on: fullyQualifiedFilename) in: [ : fd | fd containingDirectory entryAt: fd localName ]! !

!FileDirectory methodsFor: '*ma-core' stamp: 'cmm 2/20/2009 16:02'!
asFileDirectory
	^ self! !

!FileDirectory methodsFor: '*ma-core' stamp: 'cmm 4/4/2007 09:39'!
assureAbsence
	self exists ifTrue: [ self recursiveDelete ]! !

!FileDirectory methodsFor: '*ma-core' stamp: 'cmm 2/18/2009 15:03'!
directoryName
	"Answer the directory to shell to for this item."
	^self pathName! !

!FileDirectory methodsFor: '*ma-core' stamp: 'cmm 9/19/2007 16:02'!
directoryTreeDo: oneArgBlock 
	"For each file and directory in my tree, value oneArgBlock with an OrderedCollection of the path of DirectoryEntry's leading to the current node.  The first element in the collection will be the DirectoryEntryDirectory for myself, the last is the currentNode (a DirectoryEntry)."
	|myEntry|
	myEntry := OrderedCollection with: self directoryEntry.
	oneArgBlock value: myEntry.
	self 
		directoryTreeDo: oneArgBlock
		entries: myEntry! !

!FileDirectory methodsFor: '*ma-core' stamp: 'cmm 12/21/2007 15:48'!
directoryTreeDo: oneArgBlock entries: entriesCollection 
	"Value oneArgBlock with the path (an OrderedCollection of FileDirectory's) to each DirectoryEntry and the DirectoryEntry itself."
	self entries do: 
		[ : each | 
		entriesCollection add: each.
		oneArgBlock value: entriesCollection.
		each isDirectory ifTrue: 
			[ | subdir |
			subdir := each asFileDirectory.
			subdir 
				directoryTreeDo: oneArgBlock
				entries: entriesCollection ].
		entriesCollection removeLast ]! !

!FileDirectory methodsFor: '*ma-core' stamp: 'cmm 3/23/2007 12:12'!
maCopyFileNamed: sourceFilename toDirectory: aFileDirectory 
	| sourceStream |
	sourceStream := (self readOnlyFileNamed: sourceFilename) binary.
	^ aFileDirectory 
		putFile: sourceStream
		named: (aFileDirectory localNameFor: sourceFilename)! !

!FileDirectory methodsFor: '*ma-core' stamp: 'cmm 3/19/2009 22:10'!
maDetectFile: whereBlock ifFound: foundBlock ifNone: noneBlock 
	^ (self maFindEntries: whereBlock) 
		ifEmpty: noneBlock
		ifNotEmptyDo: [ : results | foundBlock value: results anyOne ]! !

!FileDirectory methodsFor: '*ma-core' stamp: 'cmm 4/8/2002 20:52'!
maEntriesMatching: pat
	"FileDirectory default fileNamesMatching: '*'"

	^ self entries select: [ :each | pat match: each name ]
! !

!FileDirectory methodsFor: '*ma-core' stamp: 'cmm 4/21/2009 13:45'!
maFileNamed: aString do: oneArgBlock 
	"A convenient accessing method for accessing a file without having to open and close yourself.  The oneArgBlock argument is the fileStream."
	| fileStream |
	fileStream := StandardFileStream fileNamed: (self fullNameFor: aString).
	fileStream ifNil: [ MaEnvironmentError signal: 'Could not open ' , aString , ' in use?' ].
	^ [ oneArgBlock value: fileStream ] ensure: [ fileStream close ]! !

!FileDirectory methodsFor: '*ma-core' stamp: 'cmm 12/4/2007 22:44'!
maFindEntries: oneArgBlock 
	"Traverse my directory and answer an OrderedCollection of DirectoryEntry's whose path of DirectoryEntry's causes oneArgBlock to value true."
	| answer |
	answer := OrderedCollection new.
	self directoryTreeDo: [ : path | (oneArgBlock value: path) ifTrue: [ answer add: path last ] ].
	^ answer! !

!FileDirectory methodsFor: '*ma-core' stamp: 'cmm 9/20/2007 17:31'!
maFindEntriesMatching: substring 
	"Traverse my directory and answer an OrderedCollection of DirectoryEntry's whose names contain substring."
	"This method focuses on user-convenience; eschewing complex wild-card matching in favor of 'type-and-go'.  But without punishing engineers who still want to use wild-card matching."
	| usePatternMatch |
	usePatternMatch := substring includesAnyOf: #($* $# ).
	^ self maFindEntries: 
		[ : path | 
		path last name in: 
			[ : theName | 
			usePatternMatch 
				ifTrue: [ substring match: theName ]
				ifFalse: 
					[ theName 
						includesSubstring: substring
						caseSensitive: false ] ] ]! !

!FileDirectory methodsFor: '*ma-core' stamp: 'cmm 4/18/2010 23:04'!
maNameFor: filename relativeTo: containingDirectory 
	"Assume that I am contained somewhere in the tree of containingDirectory.  Answer the semi-qualified name for filename, qualified from containingDirectory down to this location."
	| fullName |
	fullName := self fullNameFor: filename.
	^ fullName 
		copyFrom: containingDirectory pathName size + 2
		to: fullName size! !

!SharedQueue methodsFor: '*ma-core' stamp: 'cmm 2/8/2005 23:34'!
notEmpty

	^ self isEmpty not! !

!Process methodsFor: '*ma-core' stamp: 'cmm 11/10/2010 14:19'!
everStarted
	^ self isSuspended not or: [ self isTerminated ]! !

Notification subclass: #MaNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Exceptions'!

!MaNotification methodsFor: 'as yet unclassified' stamp: 'cmm 11/28/2011 19:40'!
messageText

	^super messageText ifNil: [ self species name ]! !

MaNotification subclass: #MagmaSessionRequest
	instanceVariableNames: 'doBlock'
	classVariableNames: ''
	poolDictionaries: 'MaCoreConstants'
	category: 'Ma-Core-Requests'!

!MagmaSessionRequest commentStamp: 'cmm 1/13/2005 23:08' prior: 0!
I am used to signal to the session controller a session directive, such as connect, commit, abort, disconnect, or others.

Use my class-side api in your domain code.  For example, in your setters:

	name: aString
		MagmaSessionRequest signalCommit: [ name _ aString ]

When Magma is not used, the session request has no effect.!

!MagmaSessionRequest class methodsFor: 'private' stamp: 'cmm 4/22/2003 23:41'!
signal: aString do: oneArgBlock
	"oneArgBlock takes the session that you wish to do something to."

	^self new
		messageText: aString;
		doBlock: oneArgBlock;
		signal! !

!MagmaSessionRequest class methodsFor: 'signaling' stamp: 'cmm 4/24/2003 15:00'!
signalAbort

	self 
		signal: 'aborting transaction'
		do: [ :session | session abort ]! !

!MagmaSessionRequest class methodsFor: 'signaling' stamp: 'cmm 4/22/2003 23:42'!
signalBegin

	self 
		signal: 'beginning transaction'
		do: [ :session | session begin ]! !

!MagmaSessionRequest class methodsFor: 'signaling' stamp: 'cmm 4/22/2003 23:43'!
signalCommit

	self 
		signal: 'committing transaction'
		do: [ :session | session commit ]! !

!MagmaSessionRequest class methodsFor: 'signaling' stamp: 'cmm 6/22/2012 13:08'!
signalCommit: aBlock 
	| result |
	self signalBegin.
	result := HasMagma
		ifTrue: [ aBlock valueWhileObservingStateChange ]
		ifFalse: [ aBlock value ].
	self signalCommit.
	^ result! !

!MagmaSessionRequest class methodsFor: 'signaling' stamp: 'cmm 4/22/2003 23:50'!
signalCommitAndBegin

	self 
		signal: 'committing transaction and beginning a new one'
		do: [ :session | session commitAndBegin ]! !

!MagmaSessionRequest class methodsFor: 'signaling' stamp: 'cmm 9/1/2010 13:09'!
signalNewGlobalReadStrategy: aMaReadStrategy
	self 
		signal: 'setting global read-strategy'
		do: [ :session | session globalReadStrategy: aMaReadStrategy ]! !

!MagmaSessionRequest class methodsFor: 'signaling' stamp: 'cmm 7/17/2010 17:56'!
signalNewReadStrategy: aMaReadStrategy
	self 
		signal: 'setting read-strategy'
		do: [ :session | session readStrategy: aMaReadStrategy ]! !

!MagmaSessionRequest class methodsFor: 'signaling' stamp: 'cmm 8/2/2009 17:06'!
signalNoteOldKeysFor: anObject 
	self 
		signal: 'noting old keys'
		do: [ : session | session noteOldKeysFor: anObject ]! !

!MagmaSessionRequest class methodsFor: 'signaling' stamp: 'cmm 11/18/2010 18:21'!
signalRefresh
	self
		signal: 'refreshing repository view'
		do: [ : session | session refresh ]! !

!MagmaSessionRequest class methodsFor: 'signaling' stamp: 'cmm 10/13/2012 23:00'!
signalSetRetrySeconds: anInteger
	self 
		signal: 'setting retrySeconds to ', anInteger
		do: [ :session | session retrySeconds: anInteger ]! !

!MagmaSessionRequest class methodsFor: 'signaling' stamp: 'cmm 7/4/2012 14:33'!
signalSetTimeoutSeconds: anInteger
	self 
		signal: 'setting timeout to ', anInteger, ' seconds'
		do: [ :session | session timeoutSeconds: anInteger ]! !

!MagmaSessionRequest class methodsFor: 'signaling' stamp: 'cmm 8/2/2009 17:06'!
signalStubOut: anObject 
	self 
		signal: 'stub out'
		do: [ : session | session stubOut: anObject ]! !

!MagmaSessionRequest methodsFor: 'private' stamp: 'cmm 3/20/2003 01:05'!
doBlock: oneArgBlock

	doBlock := oneArgBlock! !

!MagmaSessionRequest methodsFor: 'handler' stamp: 'cmm 9/8/2012 17:58'!
handleAndResumeUsing: aMagmaSession 
	self resume: (self handleUsing: aMagmaSession)! !

!MagmaSessionRequest methodsFor: 'handler' stamp: 'cmm 9/8/2012 17:57'!
handleUsing: aMagmaSession 
	aMagmaSession ifNotNil: [ doBlock value: aMagmaSession ]! !

!False methodsFor: '*ma-core' stamp: 'cmm 5/23/2012 20:34'!
hash
	"A value hash for false, so that it can be distinct from its #identityHash (which changed in 2009, causing its #hash to change too)."
	^ 826277888! !

!MethodReference methodsFor: '*ma-core' stamp: 'cmm 12/3/2008 10:40'!
maActualMethod

	| class |
	^(class := self actualClass)
		ifNil: [ nil ]
		ifNotNil:
			[ (class canUnderstand: self selector)
				ifTrue: [ class >> self selector ]
				ifFalse: [ nil ] ]! !

!MethodReference methodsFor: '*ma-core' stamp: 'cmm 12/4/2008 19:38'!
maAuthor
	"Answer the author initials of the current version of this method."
	^ self maActualMethod ifNotNilDo: 
		[ : cm | 
		cm timeStamp subStrings ifNotEmptyDo: [ : strings | strings first ] ]! !

!MethodReference methodsFor: '*ma-core' stamp: 'cmm 8/19/2003 23:53'!
maCategory
	^ self actualClass organization categoryOfElement: methodSymbol! !

!MethodReference methodsFor: '*ma-core' stamp: 'cmm 12/3/2008 00:00'!
maClassHierarchy
	^ self actualClass withAllSuperclasses reversed! !

!MethodReference methodsFor: '*ma-core' stamp: 'cmm 12/3/2008 19:07'!
maImplementors
	"Answer a collection of MethodReferences with the same selector as myself."
	^ self systemNavigation allImplementorsOf: self selector! !

!MethodReference methodsFor: '*ma-core' stamp: 'cmm 12/3/2008 19:09'!
maMethodHierarchy
	"Answer an OrderedCollection of methods in my class-hierarchy with the same selector as mine."
	| superclasses answer |
	answer := OrderedCollection new.
	superclasses := self actualClass allSuperclasses reverse.
	superclasses do: 
		[ : each | 
		(each includesSelector: self selector) ifTrue: [ answer add: (each >> self selector) methodReference ] ].
	self actualClass 
		allSubclassesWithLevelDo: 
			[ : eachClass : level | 
			(eachClass includesSelector: self selector) ifTrue: [ answer add: (eachClass >> self selector) methodReference ] ]
		startingLevel: 0.
	^ answer! !

!MethodReference methodsFor: '*ma-core' stamp: 'cmm 3/8/2011 20:01'!
maPackageInfo
	"Answer the PackageInfo containing this method."
	| methodCategory classCategory |
	methodCategory := self maCategory.
	classCategory := methodCategory first = $* ifFalse: [ self actualClass theNonMetaClass category ].
	^ PackageInfo allPackages 
		detect: 
			[ : each | 
			"detect: [ : each | each methods includes: self ]" "<-- too slow"
			(each isYourClassExtension: methodCategory) or: 
				[ classCategory notNil and: [ each systemCategories includes: classCategory ] ] ]
		ifFound: [ : foundPackage | PackageInfo named: foundPackage packageName ]
		ifNone: [ nil ]! !

!MethodReference methodsFor: '*ma-core' stamp: 'cmm 12/3/2008 17:46'!
maSenders
	"Answer a collection of MethodReferences that send my selector."
	^ self systemNavigation allCallsOn: self selector! !

!MethodReference methodsFor: '*ma-core' stamp: 'cmm 12/3/2008 17:46'!
maSentMessages
	"Answer a Collection of MethodReferences possibly called by this method."
	^ self maSentSelectors 
		inject: OrderedCollection new
		into: 
			[ : oc : each | 
			oc
				addAll: (self systemNavigation allImplementorsOf: each) ;
				yourself ]! !

!MethodReference methodsFor: '*ma-core' stamp: 'cmm 12/2/2008 23:38'!
maSentSelectors
	^ self maActualMethod 
		ifNil: 
			[ #() ]
		ifNotNilDo: [ : cm | cm messages ]! !

!MethodReference methodsFor: '*ma-core' stamp: 'cmm 12/4/2008 19:37'!
maTimeStamp
	"Answer the author initials of the current version of this method."
	^ self maActualMethod ifNotNilDo: 
		[ : cm | 
		cm timeStamp subStrings ifNotEmptyDo: 
			[ : strings | 
			DateAndTime 
				date: strings second asDate
				time: strings third asTime ] ]! !

!MethodReference methodsFor: '*ma-core' stamp: 'cmm 3/8/2011 20:02'!
packageInfo
	"Answer the PackageInfo containing this method."
	| methodCategory classCategory |
	methodCategory := self maCategory.
	classCategory := methodCategory first = $* ifFalse: [ self actualClass theNonMetaClass category ].
	^ PackageInfo allPackages 
		detect: 
			[ : each | 
			"detect: [ : each | each methods includes: self ]" "<-- too slow"
			(each isYourClassExtension: methodCategory) or: 
				[ classCategory notNil and: [ each systemCategories includes: classCategory ] ] ]
		ifFound: [ : foundPackage | PackageInfo named: foundPackage packageName ]
		ifNone: [ nil ]! !

!True methodsFor: '*ma-core' stamp: 'cmm 5/23/2012 20:34'!
hash
	"A value hash for true, so that it can be distinct from its #identityHash (which changed in 2009, causing its hash to change too)."
	^ 773324800! !

!ServerDirectory methodsFor: '*ma-core' stamp: 'cmm 4/4/2007 09:43'!
assureAbsence
	self exists ifTrue: [ self recursiveDelete ]! !

!ServerDirectory methodsFor: '*ma-core' stamp: 'cmm 4/4/2007 17:20'!
maCopyFileNamed: sourceFilename toDirectory: aFileDirectory 
	| sourceStream |
	sourceStream := (self readOnlyFileNamed: sourceFilename) binary.
	^ [ aFileDirectory 
		putFile: sourceStream
		named: (aFileDirectory localNameFor: sourceFilename) ]

		ensure: [ sourceStream close ]! !

!Stream methodsFor: '*ma-core' stamp: 'cmm 7/9/2008 21:00'!
ensureOpen
	"Base squeak assumes the need for some releasing FileStream compatibility.  However, the initializing compatibility was overlooked.  This provides some."
	^ self! !

!Stream methodsFor: '*ma-core' stamp: 'cmm 7/9/2008 21:00'!
open
	"Base squeak assumes the need for some releasing FileStream compatibility.  However, the initializing compatibility was overlooked.  This provides some."
	^ self! !

!BlockContext methodsFor: '*ma-core' stamp: 'cmm 3/23/2003 22:40'!
maOn: exc1 do: block1 on: exc2 do: block2

	^[
		[ self value ]
			on: exc1
			do: block1 ]

		on: exc2
		do: block2! !

!BlockContext methodsFor: '*ma-core' stamp: 'cmm 3/23/2003 22:43'!
maOn: exc1 do: block1 on: exc2 do: block2 on: exc3 do: block3

	^[
		[ self value ]
			on: exc1
			do: block1 ]

		maOn: exc2
		do: block2

		on: exc3
		do: block3! !

!Mutex methodsFor: '*ma-core' stamp: 'cmm 7/5/2012 10:12'!
isCritical
	"Answer whether I am guarding execution of a piece of code at the moment."
self maMarked: 'delete'.
	^ semaphore isSignaled not! !

!DirectoryEntryDirectory methodsFor: '*ma-core-private' stamp: 'cmm 9/13/2007 17:24'!
addToDirectoryCount: anInteger 
	self directoryCountStat addValue: anInteger! !

!DirectoryEntryDirectory methodsFor: '*ma-core-private' stamp: 'cmm 9/13/2007 17:24'!
addToFileCount: anInteger 
	self fileCountStat addValue: anInteger! !

!DirectoryEntryDirectory methodsFor: '*ma-core-private' stamp: 'cmm 9/13/2007 17:19'!
addToTotalSize: anInteger 
	self fileSizeStat addValue: anInteger! !

!DirectoryEntryDirectory methodsFor: '*ma-core-private' stamp: 'cmm 9/13/2007 17:54'!
contributeStatisticsTo: aDirectoryEntryDirectory 
	"Add my size information to the cumulative total in aDirectoryEntryDirectory."
	self initializeStatistics.
	aDirectoryEntryDirectory
		addToTotalSize: self totalFileSize ;
		addToDirectoryCount: self directoryCount ;
		addToFileCount: self fileCount! !

!DirectoryEntryDirectory methodsFor: '*ma-core' stamp: 'cmm 9/19/2007 15:41'!
directoryCount
	"Answer the total number of directory entries in my directory, including all subdirectories."
	^ self directoryCountStat sum! !

!DirectoryEntryDirectory methodsFor: '*ma-core' stamp: 'cmm 9/28/2007 12:33'!
directoryCountStat
	"Statistics about the number of directories per folder."
	^ self 
		attributeNamed: #directoryCountStat
		ifAbsent: 
			[ self
				initializeStatistics ;
				directoryCountStat ]! !

!DirectoryEntryDirectory methodsFor: '*ma-core-private' stamp: 'cmm 9/13/2007 17:22'!
directoryCountStat: aMaStat 
	self 
		attributeNamed: #directoryCountStat
		put: aMaStat! !

!DirectoryEntryDirectory methodsFor: '*ma-core' stamp: 'cmm 2/12/2009 18:56'!
directoryName
	"Answer the directory to shell to for this item."
	^ self asFileDirectory pathName! !

!DirectoryEntryDirectory methodsFor: '*ma-core' stamp: 'cmm 9/19/2007 15:41'!
fileCount
	"Answer the total number of file entries in my directory, including all subdirectories."
	^ self fileCountStat sum! !

!DirectoryEntryDirectory methodsFor: '*ma-core' stamp: 'cmm 9/28/2007 12:32'!
fileCountStat
	"Statistics about the number of files per folder."
	^ self 
		attributeNamed: #fileCountStat
		ifAbsent: 
			[ self
				initializeStatistics ;
				fileCountStat ]! !

!DirectoryEntryDirectory methodsFor: '*ma-core-private' stamp: 'cmm 9/13/2007 17:21'!
fileCountStat: aMaStat 
	self 
		attributeNamed: #fileCountStat
		put: aMaStat! !

!DirectoryEntryDirectory methodsFor: '*ma-core' stamp: 'cmm 9/13/2007 17:19'!
fileSizeStat
	^ self 
		attributeNamed: #fileSizeStat
		ifAbsent: 
			[ self
				initializeStatistics ;
				fileSizeStat ]! !

!DirectoryEntryDirectory methodsFor: '*ma-core-private' stamp: 'cmm 9/13/2007 17:19'!
fileSizeStat: aMaStat 
	self 
		attributeNamed: #fileSizeStat
		put: aMaStat! !

!DirectoryEntryDirectory methodsFor: '*ma-core-private' stamp: 'cmm 9/13/2007 17:48'!
incrementDirectoryOrFileCount: twoElementArray 
	twoElementArray 
		at: 1
		put: (twoElementArray at: 1) + 1! !

!DirectoryEntryDirectory methodsFor: '*ma-core-private' stamp: 'cmm 9/13/2007 17:51'!
initializeStatistics
	"Enumerate my entire directory tree to gather my statistics."
	| directoryAndFileCount |
	self
		fileSizeStat: MaStat new ;
		fileCountStat: MaStat new ;
		directoryCountStat: MaStat new.
	directoryAndFileCount := {  0. 0  }.
	self asFileDirectory entries in: 
		[ : myEntries | 
		myEntries
			do: [ : eachEntry | eachEntry contributeStatisticsTo: self ] ;
			do: [ : eachEntry | eachEntry incrementDirectoryOrFileCount: directoryAndFileCount ] ].
	self
		addToDirectoryCount: (directoryAndFileCount at: 1) ;
		addToFileCount: (directoryAndFileCount at: 2)! !

!DirectoryEntryDirectory methodsFor: '*ma-core' stamp: 'cmm 9/13/2007 17:19'!
totalFileSize
	"Answer the cumulative size of all files and folders in the directories I contain, including their totalFileSize."
	^ self fileSizeStat sum ifNil: [ 0 ]! !

!Month methodsFor: '*ma-core' stamp: 'cmm 8/3/2010 23:04'!
maPrintAbbreviatedOn: aStream 
	aStream
		 maPrint: self year asString ;
		 maPrint: $- ;
		 maPrint:
			(self monthIndex
				printPaddedWith: $0
				to: 2)! !

!Month methodsFor: '*ma-core' stamp: 'cmm 8/3/2010 23:33'!
maSuggestedFilename
	"Often, it is useful for month filenames to be sortable, therefore override the standard maSuggestedFilename which would otherwise answer, 'July', 'August', etc."
	^ String streamContents: [ : stream | self maPrintAbbreviatedOn: stream ]! !

!ZipArchive methodsFor: '*ma-core' stamp: 'cmm 10/30/2009 16:36'!
filesDo: aBlock tempDir: aFileDirectory 
	"Value aBlock with each DirectoryEntryFile in the receiver, after extracting it to aFileDirectory.  aFileDirectory is deleted after all members are valued."
	[ self extractAllTo: aFileDirectory ] ensure: [ self close ].
	
	[ self members do: [ : each | aBlock value: (aFileDirectory entryAt: each localFileName) ] ] ensure: 
		[ self members do: [ : each | aFileDirectory deleteFileNamed: each localFileName ] ]! !

!TimeProfileBrowser class methodsFor: '*ma-core' stamp: 'cmm 12/24/2003 13:25'!
maOnBlock: block if: conditionBlock
	"Allows you to build up your sample set before invoking the profiler."
	^conditionBlock value 
		ifTrue: [ self onBlock: block ]
		ifFalse: [ block value ]! !

!Exception class methodsFor: '*ma-core' stamp: 'cmm 9/18/2005 23:56'!
messageText: aString

	^ self new
		messageText: aString ;
		yourself! !

!Exception methodsFor: '*ma-core' stamp: 'cmm 9/28/2008 18:18'!
isMaUserError

	^false! !

!Exception methodsFor: '*ma-core' stamp: 'cmm 8/11/2002 23:35'!
maIsException

	^true! !

Exception subclass: #MaRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Requests'!

MaRequest subclass: #MaStringRequest
	instanceVariableNames: 'initialAnswer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Requests'!

!MaStringRequest commentStamp: 'cmm 10/9/2005 12:01' prior: 0!
Some applications need to prompt the user for a String at an inopportune time; for example some operations may require the users privateKey file and we want to do this on a JIT basis.

The purpose of this class is to provide a default way to ask the user for a String while allowing programs to override with their own approach.!

MaStringRequest subclass: #MaFileRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Requests'!

!MaFileRequest class methodsFor: 'create' stamp: 'cmm 11/16/2005 22:11'!
message: aString filename: filenameString initialDirectory: aFileDirectory

	^ self new
		messageText: aString, '.  Where is ', filenameString, '?' ;
		initialAnswer: aFileDirectory fullName ;
		yourself! !

!MaStringRequest class methodsFor: 'create' stamp: 'cmm 9/27/2005 00:49'!
messageText: messageString initialAnswer: initialAnswerString

	^ (self messageText: messageString) 
		initialAnswer: initialAnswerString ;
		yourself! !

!MaStringRequest methodsFor: 'initialize-release' stamp: 'cmm 12/7/2011 21:49'!
defaultAction
	self resume:
		(UIManager default 
			request: self messageText
			initialAnswer: self initialAnswer)! !

!MaStringRequest methodsFor: 'accessing' stamp: 'cmm 3/1/2012 21:30'!
initialAnswer
	^ initialAnswer ifNil: [ String empty ]! !

!MaStringRequest methodsFor: 'initialize-release' stamp: 'cmm 9/27/2005 00:49'!
initialAnswer: aString
	initialAnswer := aString! !

!Date methodsFor: '*ma-core' stamp: 'cmm 7/29/2008 23:14'!
maAddYears: anInteger 
	"Answer a new date anInteger years after or, if anInteger is negative, before, the receiver."
	^ Date 
		newDay: self dayOfMonth
		month: self monthIndex
		year: self year + anInteger! !

!Date methodsFor: '*ma-core' stamp: 'cmm 6/17/2010 15:29'!
maSuggestedFilename 
	^ self yyyymmdd asFileName! !

!Point methodsFor: '*ma-core' stamp: 'cmm 4/9/2010 12:53'!
maCloseTo: aPoint 
	^ (x closeTo: aPoint x) and: [ y closeTo: aPoint y ]! !

!Point methodsFor: '*ma-core' stamp: 'cmm 6/29/2010 22:46'!
maHasZero
	^ x isZero or: [ y isZero ]! !

!Point methodsFor: '*ma-core' stamp: 'cmm 8/31/2010 13:34'!
maLongestSide
	^ x max: y! !

!PackageInfo class methodsFor: '*ma-core' stamp: 'cmm 3/8/2011 19:59'!
maPackages
	^ (self allPackages select: 
		[ : each | 
		(each packageName 
			maBeginsWith: 'Ma'
			caseSensitive: true) or: 
			[ {  'WriteBarrier'. 'Collections-BTree'  } anySatisfy: [ : eachAlt | eachAlt = each packageName ] ] ]) collect: [ : each | PackageInfo named: each packageName ]! !

!PackageInfo methodsFor: '*ma-core' stamp: 'cmm 4/11/2009 18:25'!
calculatePrerequisitesUsing: interestingPackages 
	"Prerequisite calculation takes so long it is calculated separately by the developer, and then the #prerequisitePackageNames is manually overridden."
	| map |
	map := self class prerequisiteMapFor: interestingPackages.
	^ self calculatePrerequisitesUsingMap: map! !

!PackageInfo methodsFor: '*ma-core' stamp: 'cmm 3/20/2009 16:26'!
calculatePrerequisitesUsingMap: map 
	^ map at: self! !

!PackageInfo methodsFor: '*ma-core' stamp: 'cmm 3/20/2009 17:58'!
dependenciesFrom: anotherPackage 
	| answer |
	answer := OrderedCollection new.
	self externalUsers do: 
		[ : each | 
		each key maPackageInfo ifNotNilDo: [ : pi | pi = anotherPackage ifTrue: [ answer add: each ] ] ].
	self externalSubclasses do: 
		[ : each | 
		each maPackageInfo ifNotNilDo: [ : pi | pi = anotherPackage ifTrue: [ answer add: each ] ] ].
	^ answer! !

!PackageInfo methodsFor: '*ma-core' stamp: 'cmm 3/20/2009 15:02'!
dependentPackageInfos
	| answer |
	answer := Set new.
	self externalUsers do: [ : each | each key maPackageInfo ifNotNilDo: [ : pi | answer add: pi ] ].
	self externalSubclasses do: [ : each | each maPackageInfo ifNotNilDo: [ : pi | answer add: pi ] ].
	^ answer asArray! !

!PackageInfo methodsFor: '*ma-core' stamp: 'cmm 3/18/2009 21:10'!
localRepositories
	^ self repositories select: [ : each | each isDirectoryRepository ]! !

!PackageInfo methodsFor: '*ma-core' stamp: 'cmm 3/21/2009 18:22'!
packageNames
	"My name and the names of the all direct and indirect prerequistes."
	^ self allPrerequisitePackageNames
		add: self packageName ;
		yourself! !

!PackageInfo methodsFor: '*ma-core' stamp: 'cmm 3/17/2009 10:22'!
patchNames
	"Answer a collection of change-set patches required for this package."
	^ #()! !

!PackageInfo methodsFor: '*ma-core' stamp: 'cmm 3/18/2009 21:06'!
repositories
	^ self workingCopy repositoryGroup repositories! !

!PackageInfo methodsFor: '*ma-core' stamp: 'cmm 11/9/2010 22:48'!
resourceNames
	"Answer a collection of files to load after the code has been loaded."
	^ #()! !

!PackageInfo methodsFor: '*ma-core' stamp: 'cmm 3/20/2009 18:02'!
whyNeeded: anotherPackage 
	| answer |
	answer := OrderedCollection new.
	anotherPackage externalUsers do: 
		[ : each | 
		each key maPackageInfo ifNotNilDo: [ : pi | pi = self ifTrue: [ answer add: each ] ] ].
	anotherPackage externalSubclasses do: 
		[ : each | 
		each maPackageInfo ifNotNilDo: [ : pi | pi = self ifTrue: [ answer add: each ] ] ].
	^ answer! !

!LargePositiveInteger methodsFor: '*ma-core' stamp: 'cmm 6/13/2010 23:03'!
maLog: anInteger 
	"Answer the log base aNumber of the receiver."
	^ (self log: anInteger) in:
		[ : firstAttempt | firstAttempt = Float infinity
			ifTrue:
				[ | m |
				"Search for it."
				m := 1023.
				(anInteger raisedTo: m) > self
					ifTrue:
						[ [ (anInteger raisedTo: m) < self ] whileFalse: [ m := m - 1 ].
						(m - 1) asFloat ]
					ifFalse:
						[ [ (anInteger raisedTo: m) > self ] whileFalse: [ m := m + 1 ].
						(m - 1) asFloat ] ]
			ifFalse: [ firstAttempt ] ]! !

!Dictionary methodsFor: '*ma-core' stamp: ''!
maAllKeysAtValue: anObject

	| aCollection |

	aCollection := OrderedCollection new.
	self keysAndValuesDo: [ :key :value | value = anObject ifTrue: [ aCollection add: key ] ].
	^aCollection! !

!Dictionary methodsFor: '*ma-core' stamp: 'cmm 10/27/2010 20:49'!
maBindingAtRandom
	| binding | binding := nil.
	[ binding isNil ] whileTrue: [ binding := array at: (1 to: array size) atRandom ].
	^ binding! !

!Dictionary methodsFor: '*ma-core' stamp: 'cmm 10/27/2010 20:50'!
maKeyAtRandom
	^ self maBindingAtRandom key! !

!Dictionary methodsFor: '*ma-core' stamp: 'cmm 7/24/2002 20:45'!
maShallowCopy

	| copy |
	copy := self species new.  "new: seems buggy w/ Dictionaries.."
	self keysAndValuesDo: 
		[ :eachKey :eachValue | copy at: eachKey put: eachValue ].
	^copy! !

!ByteString methodsFor: '*ma-core' stamp: 'cmm 11/10/2005 20:48'!
maAlphabeticalNext
	"Answer the next higher String.  This is useful in indexing systems to search for left-side matching strings."
	| nextValue stream |
	nextValue := (self asByteArray inject: 0 into: [ :sum :each | sum * 256 + each ]) + 1.
	stream := WriteStream on: ByteArray new.
	nextValue digitLength to: 1 by: -1 do:
		[ : digitIndex |
		stream nextPut: (nextValue digitAt: digitIndex) ].
	^ stream contents asString! !

!Class methodsFor: '*ma-core' stamp: 'cmm 5/2/2011 14:34'!
packageInfo
	^ PackageInfo allPackages
		detect: [ : each | each includesClass: self ]
		ifNone: [ nil ]! !

!Class methodsFor: '*ma-core' stamp: 'cmm 6/14/2012 15:58'!
theNonWriteBarrierClass
	^ self! !

Object subclass: #MaObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Abstractions'!

!MaObject commentStamp: 'cmm 10/25/2005 22:43' prior: 0!
I provide automatic initialization (which was later added to Squeak 3.7), and the notion of a verbose (standard printOn:) and concise (maPrintAbbreviatedOn:) forms for brevity in collections.!

MaObject subclass: #MaAttributableObject
	instanceVariableNames: 'attributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Abstractions'!

!MaAttributableObject commentStamp: 'cmm 10/7/2009 21:03' prior: 0!
Subclass from this abstract class to inherit an attribute Dictionary.!

!MaAttributableObject methodsFor: 'attributes' stamp: 'cmm 4/9/2010 14:43'!
attributeNamed: aString 
	^ attributes ifNotNil: 
		[ attributes 
			at: aString
			ifAbsent: [ nil ] ]! !

!MaAttributableObject methodsFor: 'attributes' stamp: 'cmm 6/20/2012 17:36'!
attributeNamed: aString put: anObject 
	anObject ifNotNil: [ attributes ifNil: [ attributes := Dictionary new ] ].
	^ anObject
		ifNil: [ self removeAttributeNamed: aString ]
		ifNotNil:
			[ attributes
				at: aString
				put: anObject ]! !

!MaAttributableObject methodsFor: 'copying' stamp: 'cmm 1/16/2012 16:39'!
attributesToDeepCopy
	"Subclasses override."
	^ attributes
		ifNil: [ Array empty ]
		ifNotNil: [ attributes keys ]! !

!MaAttributableObject methodsFor: 'attributes' stamp: 'cmm 4/9/2010 14:42'!
hasAttributeNamed: aString 
	^ attributes 
		ifNil: [ false ]
		ifNotNil: [ attributes includesKey: aString ]! !

!MaAttributableObject methodsFor: 'remember/restore' stamp: 'cmm 12/7/2009 17:02'!
rememberAttributes
	"Remember my current attributes as an attribute called, #priorAttributes."
	| priorAttributes |
	(self attributeNamed: #priorAttributes) ifNotNil: [ MaUserError signal: 'priorAttributes already present' ].
	priorAttributes := attributes.
	attributes := attributes copy.
	self 
		attributeNamed: #priorAttributes
		put: priorAttributes! !

!MaAttributableObject methodsFor: 'attributes' stamp: 'cmm 10/7/2009 21:02'!
removeAttributeNamed: aString 
	^ attributes ifNotNil: 
		[ | answer |
		answer := attributes 
			removeKey: aString
			ifAbsent: [ nil ].
		attributes ifEmpty: [ attributes := nil ].
		answer ]! !

!MaAttributableObject methodsFor: 'remember/restore' stamp: 'cmm 10/28/2011 15:38'!
restoreAttributes
	"Restore my prior-attributes which were remembered via #rememberAttributes."
	attributes 
		at: #priorAttributes
		ifPresent: [ : foundAttributes | attributes := foundAttributes ]
		ifAbsent: 
			[ "there were no attributes when rememberAttributes was called"
			 ]! !

!MaAttributableObject methodsFor: 'copying' stamp: 'cmm 2/16/2012 16:56'!
veryDeepInner: aDeepCopier 
	super veryDeepInner: aDeepCopier.
	attributes := attributes copy.
	self attributesToDeepCopy do:
		[ : eachAttribute | attributes
			at: eachAttribute
			put: ((attributes at: eachAttribute) veryDeepCopyWith: aDeepCopier) ]! !

MaObject subclass: #MaCommandLineProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Utilities'!

!MaCommandLineProcessor commentStamp: 'cmm 5/25/2010 12:37' prior: 0!
MaCommandLineProcessor is a place for Smalltalk scripts to be called via command-line.!

!MaCommandLineProcessor class methodsFor: 'api' stamp: 'cmm 5/25/2010 12:45'!
args
	"Answer an Array of the command-line arguments passed in."
	| stream |
	stream := WriteStream on: Array new.
	3
		to: 1000
		do:
			[ : n | (SmalltalkImage current getSystemAttribute: n)
				ifNil: [ ^ stream contents ]
				ifNotNilDo:
					[ : arg | stream nextPut: arg ] ]! !

!MaCommandLineProcessor class methodsFor: 'api' stamp: 'cmm 10/14/2012 22:05'!
do: aBlock 
	| stdOut stdErr |
	self ensureStartedUp.
	stdOut := StandardFileStream stdout.
	stdErr := StandardFileStream stderr.
	[ [aBlock valueWithAllArguments: self args ] on: ProgressInitiationException do: [ : pie | pie defaultAction ] ]
		on: Notification , Warning
		do:
			[ : noti | stdOut
				 nextPutAll: DateAndTime now asString ;
				 space ;
				 nextPutAll: noti description ;
				 perform: MultiByteFileStream lineEndDefault.
			noti resume ]
		on: SyntaxErrorNotification
		do:
			[ : err | stdOut
				 nextPutAll: err errorCode ;
				 perform: MultiByteFileStream lineEndDefault.
			self haltOrQuit ]
		on: Error
		do:
			[ : err | err isMaUserError ifFalse:
				[ stdErr nextPutAll:
					(err signalerContext longStack
						copyReplaceAll: Character cr asString
						with: (String perform: MultiByteFileStream lineEndDefault)) ].
			stdErr
				 nextPutAll: 'ERROR:  ' ;
				 nextPutAll: DateAndTime now asString ;
				 space ;
				 perform: MultiByteFileStream lineEndDefault ;
				 nextPutAll: err description ;
				 perform: MultiByteFileStream lineEndDefault.
			self haltOrQuit.
			err isResumable ifTrue: [ err resume ] ]! !

!MaCommandLineProcessor class methodsFor: 'api' stamp: 'cmm 10/30/2010 20:34'!
do: aBlock snapshotAs: imageName andQuit: shouldQuit 
	self do: aBlock.
	imageName
		ifNil:
			[ shouldQuit ifTrue:
				[ Smalltalk
					snapshot: false
					andQuit: shouldQuit ] ]
		ifNotNil:
			[ | fixedName |
			fixedName := (imageName endsWith: '.image')
				ifTrue: [ imageName ]
				ifFalse: [ imageName , '.image' ].
			(FileDirectory default fullNameFor: fixedName) = Smalltalk imageName
				ifTrue:
					[ Smalltalk
						snapshot: true
						andQuit: shouldQuit ]
				ifFalse:
					[ Smalltalk maSaveAs: imageName.
					"Avoid instant-quit when relaunching by resaving."
					shouldQuit ifTrue:
						[ Smalltalk
							snapshot: true
							andQuit: shouldQuit ] ] ]! !

!MaCommandLineProcessor class methodsFor: 'api' stamp: 'cmm 10/30/2010 20:25'!
doAndQuit: aBlock 
	self
		do: aBlock
		snapshotAs: nil
		andQuit: true! !

!MaCommandLineProcessor class methodsFor: 'private' stamp: 'cmm 8/4/2010 10:30'!
ensureStartedUp
	#(#ThisOSProcess #MagmaRepositoryController ) do: [ : each | self ensureStartedUp: each ]! !

!MaCommandLineProcessor class methodsFor: 'private' stamp: 'cmm 6/21/2010 23:22'!
ensureStartedUp: className 
	(Smalltalk hasClassNamed: className) ifTrue: [ (Smalltalk classNamed: className) startUp: true ]! !

!MaCommandLineProcessor class methodsFor: 'private' stamp: 'cmm 10/7/2010 18:03'!
haltOrQuit
	SmalltalkImage current maIsHeadless
		ifTrue: [ self quit ]
		ifFalse: [ self halt ]! !

!MaCommandLineProcessor class methodsFor: 'api' stamp: 'cmm 5/25/2010 12:39'!
quit
	SmalltalkImage current
		snapshot: false
		andQuit: true! !

!MaObject class methodsFor: 'testing' stamp: 'cmm 6/18/2011 14:25'!
hasMaui
	"Answer whether some version of Maui is installed on this system."
	^ Smalltalk hasClassNamed: #MauiWorld! !

!MaObject methodsFor: 'printing' stamp: 'cmm 11/29/2012 16:24'!
abbreviation
	^ self class abbreviation! !

!MaObject methodsFor: 'printing' stamp: 'cmm 4/22/2002 22:13'!
printOn: aStream

	super printOn: aStream.
	aStream space.
	self maPrintAbbreviatedOn: aStream
! !

MaObject subclass: #MaObjectComparison
	instanceVariableNames: 'object1 object2 comparisonOperator differences'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Utilities'!

!MaObjectComparison class methodsFor: 'create' stamp: 'cmm 11/26/2012 21:07'!
with: firstObject with: secondObject 
	^ self new
		
		setObject1: firstObject
		object2: secondObject ;
		 yourself! !

!MaObjectComparison methodsFor: 'accessing' stamp: 'cmm 11/26/2012 21:11'!
comparisonOperator
	^ comparisonOperator! !

!MaObjectComparison methodsFor: 'accessing' stamp: 'cmm 11/26/2012 21:10'!
comparisonOperator: operatorSymbol
	comparisonOperator = operatorSymbol ifFalse:
		[ comparisonOperator := operatorSymbol.
		self resetDifferences ]! !

!MaObjectComparison methodsFor: 'testing' stamp: 'cmm 12/19/2012 17:04'!
differences
	^ differences ifNil:
		[ differences := self object1
			differencesWith: self object2
			using: comparisonOperator ]! !

!MaObjectComparison methodsFor: 'testing' stamp: 'cmm 11/26/2012 21:13'!
differencesFor: differencesKey 
	^ self differences
		at: differencesKey
		ifAbsent: [ Array empty ]! !

!MaObjectComparison methodsFor: 'testing' stamp: 'cmm 11/26/2012 21:12'!
differsBy: differencesKey
	^ self differences includesKey: differencesKey! !

!MaObjectComparison methodsFor: 'testing' stamp: 'cmm 11/28/2012 09:25'!
differsByOtherThan: allowedKeys 
	"Answer whether my objects differ by attributes _besides_ the ones specified by allowedKeys."
	| allowedCollection |
	allowedCollection := allowedKeys asCollection.
	^ self differences keys anySatisfy:
		[ : each | (allowedCollection includes: each) not ]! !

!MaObjectComparison methodsFor: 'initializing' stamp: 'cmm 11/26/2012 21:16'!
initialize
	super initialize.
	comparisonOperator := #=.
	self resetDifferences! !

!MaObjectComparison methodsFor: 'accessing' stamp: 'cmm 11/26/2012 21:06'!
object1
	^ object1! !

!MaObjectComparison methodsFor: 'accessing' stamp: 'cmm 11/26/2012 21:06'!
object2
	^ object2! !

!MaObjectComparison methodsFor: 'private' stamp: 'cmm 11/26/2012 21:09'!
resetDifferences
	differences := nil! !

!MaObjectComparison methodsFor: 'initializing' stamp: 'cmm 11/26/2012 21:07'!
setObject1: firstObject object2: secondObject 
	object1 := firstObject.
	object2 := secondObject! !

MaObject subclass: #MaSingleton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Abstractions'!

MaSingleton class
	instanceVariableNames: 'current'!

MaSingleton class
	instanceVariableNames: 'current'!

!MaSingleton class methodsFor: 'accessing' stamp: 'cmm 11/23/2010 23:51'!
current
	^ current ifNil:
		[ self current: self new.
		current ]! !

!MaSingleton class methodsFor: 'accessing' stamp: 'cmm 11/18/2010 13:11'!
current: anObject
	current := anObject! !

!MaSingleton class methodsFor: 'actions' stamp: 'cmm 1/10/2005 21:54'!
initialize

	super initialize.
	self reset! !

!MaSingleton class methodsFor: 'actions' stamp: 'cmm 4/22/2011 14:31'!
reset
	self current: nil! !

MaObject subclass: #MaStat
	instanceVariableNames: 'last min max count sum'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Domains'!

!MaStat commentStamp: 'cmm 5/6/2008 12:31' prior: 0!
The name, "MaStat" is purposefully abbreviated because its capability, itself, is abbreviated.  It is not intended to ever develop into a "statistics framework", only to serve basic statistical purposes.!

!MaStat class methodsFor: 'as yet unclassified' stamp: 'cmm 12/23/2008 18:53'!
defaultPrecision
	^ 0.001! !

!MaStat class methodsFor: 'as yet unclassified' stamp: 'cmm 11/16/2004 10:51'!
example

	| stat |
	stat := self new.
	#(1 2 5 8 9 15) do: [ :each | stat addValue: each ].
	stat! !

!MaStat methodsFor: 'summarize' stamp: 'cmm 7/13/2009 18:06'!
+ aMaStat 
	| n x c s |
	n := self min 
		ifNil: [ aMaStat min ]
		ifNotNil: 
			[ aMaStat min 
				ifNil: [ self min ]
				ifNotNil: [ self min min: aMaStat min ] ].
	x := self max 
		ifNil: [ aMaStat max ]
		ifNotNil: 
			[ aMaStat max 
				ifNil: [ self max ]
				ifNotNil: [ self max max: aMaStat max ] ].
	c := self count 
		ifNil: [ aMaStat count ]
		ifNotNil: 
			[ aMaStat count 
				ifNil: [ self count ]
				ifNotNil: [ self count + aMaStat count ] ].
	s := self sum 
		ifNil: [ aMaStat sum ]
		ifNotNil: 
			[ aMaStat sum 
				ifNil: [ self sum ]
				ifNotNil: [ self sum + aMaStat sum ] ].
	^ self maOriginalClass new 
		setLast: aMaStat last
		min: n
		max: x
		count: c
		sum: s! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 5/7/2008 15:36'!
addValue: aMagnitude 
	"This is named addValue: instead of add: to be easier to find senders."
	last := aMagnitude.
	count := count + 1.
	min := min 
		ifNil: [ aMagnitude ]
		ifNotNil: [ min min: aMagnitude ].
	max := max 
		ifNil: [ aMagnitude ]
		ifNotNil: [ max max: aMagnitude ].
	sum := sum 
		ifNil: [ aMagnitude ]
		ifNotNil: [ sum + aMagnitude ].
	^ aMagnitude! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 9/28/2007 12:31'!
count
	"The number of values."
	^count! !

!MaStat methodsFor: 'initialize' stamp: 'cmm 4/28/2008 22:56'!
initialize
	super initialize.
	count := last := 0! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 9/28/2007 12:33'!
last
	"The last value recorded."
	^ last! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 11/16/2004 21:50'!
last: aMagnitude

	last := aMagnitude! !

!MaStat methodsFor: 'printing' stamp: 'cmm 11/16/2004 21:52'!
maPrintAbbreviatedOn: aStream

	self
		maPrintAttributes: #(min max sum count mean)
		on: aStream! !

!MaStat methodsFor: 'maui support' stamp: 'cmm 12/22/2008 14:04'!
mauiDefaultColumns
	^ #(#count #min #max #mean #sum #last )! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 9/27/2007 17:23'!
max
	"The highest value."
	^max! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 12/8/2009 10:36'!
mean
	"The average value."
	^ (sum isNil or: [ count isNil ]) 
		ifTrue: [ nil ]
		ifFalse: [ sum / count asFloat roundTo: self class defaultPrecision ]! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 9/27/2007 17:23'!
min
	"The minimum value."
	^min! !

!MaStat methodsFor: 'initialize' stamp: 'cmm 4/28/2008 22:29'!
setLast: newLast min: newMin max: newMax count: newCount sum: newSum 
	last := newLast.
	min := newMin.
	max := newMax.
	count := newCount.
	sum := newSum! !

!MaStat methodsFor: 'accessing' stamp: 'cmm 9/27/2007 17:24'!
sum
	"The sum of all values."
	^ sum! !

MaObject subclass: #MaTextMessage
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Utilities'!

!MaTextMessage commentStamp: '<historical>' prior: 0!
This little tool helps you build non-wimpy message strings.!

!MaTextMessage methodsFor: 'building' stamp: 'cmm 3/6/2002 22:47'!
append: aString

	self appendString: aString
! !

!MaTextMessage methodsFor: 'building' stamp: ''!
appendAndedList: aCollection

	self
		appendList: (
			aCollection
				collect: [ :each | each isString ifTrue: [ each ] ifFalse: [ each printString ] ])
		with: ' and '
! !

!MaTextMessage methodsFor: 'building' stamp: ''!
appendCount: aNumber of: aString
	"aString should be a non-plural word.  This method will append the
	plural version of aString (by simply adding 's') if aNumber is > 1."

	self
		append: aNumber;
		space.
	self
		appendProperlyPluralized: aString
		for: aNumber
! !

!MaTextMessage methodsFor: 'private initializing' stamp: ''!
appendList: aCollection with: aString

	| index |

	aCollection isEmpty ifTrue: [ ^nil ].
	index := 1.
	aCollection do:
		[ :each |
		self append: each.
		aCollection size > 1
			ifTrue:
				[ index < (aCollection size - 1)
					ifTrue:
						[ self append: ', ' ]
					ifFalse:
						[ index = (aCollection size - 1) ifTrue: [ self append: aString ] ] ].
		index := index + 1 ]
! !

!MaTextMessage methodsFor: 'building' stamp: 'cmm 11/25/2003 00:19'!
appendProperlyPluralized: aString for: aNumber
	"aString should be a non-plural word.  This method will append the
	plural version of aString (by simply adding 's') if aNumber is > 1."

	self append: aString.
	aString last asLowercase = $s
		ifTrue: [ stream nextPut: $' ]
		ifFalse: [ stream nextPut: $s ]! !

!MaTextMessage methodsFor: 'private building' stamp: 'cmm 11/25/2003 00:17'!
appendString: anObject

	anObject isString 
		ifTrue: [ stream nextPutAll: anObject ] 
		ifFalse: [ anObject printOn: stream ]! !

!MaTextMessage methodsFor: 'private initializing' stamp: 'cmm 11/25/2003 00:16'!
initialize

	super initialize.
	stream := String new writeStream! !

!MaTextMessage methodsFor: 'accessing' stamp: 'cmm 11/25/2003 00:22'!
messageText

	stream last = $. ifFalse: [ stream nextPut: $. ].
	^stream contents
! !

!MaTextMessage methodsFor: 'building' stamp: 'cmm 12/27/2008 16:24'!
space
	self append: ' '! !

!MaTextMessage methodsFor: 'private accessing' stamp: 'cmm 11/25/2003 00:16'!
string

	^stream contents! !

MaObject subclass: #MaTimer
	instanceVariableNames: 'name process monitor expired delay'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Utilities'!

!MaTimer commentStamp: 'cmm 5/2/2008 11:59' prior: 0!
A very simple class to perform a Block after a specified amount of time.  #cancel will stop the countdown.  #expired answers true if the timer expired and the do Block executed.!

!MaTimer class methodsFor: 'as yet unclassified' stamp: 'cmm 5/10/2008 13:09'!
unload
	self allInstancesDo: [ : each | each cancel ]! !

!MaTimer methodsFor: 'access' stamp: 'cmm 6/4/2008 08:55'!
cancel
	"Only allow termination while the delay is running, not while the do block is running."
	process ifNil: [ ^ self ].
	expired ifTrue: 
		[ "There's no canceling now..!!"
		^ self ].
	"Don't allow process to terminate itself."
	process == Processor activeProcess ifTrue: [ MaSoftwareError signal: 'Do not cancel from this process!!' ].
	monitor 
		ifNil: [ process terminate ]
		ifNotNil: [ monitor critical: [ process terminate ] ].
	process := nil! !

!MaTimer methodsFor: 'access' stamp: 'cmm 12/15/2008 17:42'!
expired
	"Answer whether the waiting period before executing my do block has elapsed.  It does not indicate whether the do block has executed or not."
	^ self started and: [ monitor critical: [ expired ] ]! !

!MaTimer methodsFor: 'access' stamp: 'cmm 12/19/2008 14:19'!
in: aDuration do: aBlock 
	process := 
	[ monitor := Mutex new.
	expired := false.
	delay := (Delay forDuration: aDuration) wait.
	name ifNil: [ name := 'MaTimer for ' , aDuration printString ].
	monitor critical: 
		[ expired := true.
		aBlock value ] ] newProcess.
	process
		name: self name ;
		priority: Processor userInterruptPriority-1.
	process resume! !

!MaTimer methodsFor: 'initialize' stamp: 'cmm 3/7/2008 17:08'!
initialize
	super initialize.
	expired := false! !

!MaTimer methodsFor: 'access' stamp: 'cmm 6/5/2008 22:10'!
name
	^name! !

!MaTimer methodsFor: 'access' stamp: 'cmm 6/5/2008 22:10'!
name: anObject
	name := anObject! !

!MaTimer methodsFor: 'access' stamp: 'cmm 12/15/2008 17:42'!
resetDelay
	"If I am still counting down to ding, reset my Delay timer back to the original duration.  Otherwise, do nothing.  
	This might make me useful as a countdown timer to disconnect after a period of inactivity."
	self started ifFalse: [ ^ self ].	"not yet started.."
	monitor critical: [ delay ifNotNil: [ delay maReset ] ]! !

!MaTimer methodsFor: 'access' stamp: 'cmm 12/15/2008 17:41'!
started
	^ monitor notNil! !

MaObject subclass: #MauiPage
	instanceVariableNames: 'text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Domain'!

!MauiPage class methodsFor: 'create' stamp: 'cmm 4/19/2009 16:35'!
new
	"Create a new page."
	^ super new! !

!MauiPage class methodsFor: 'create' stamp: 'cmm 11/19/2007 14:34'!
streamContents: aBlock 
	^ self text: (Text streamContents: aBlock)! !

!MauiPage class methodsFor: 'create' stamp: 'cmm 11/19/2007 14:12'!
text: aText 
	^ self new
		text: aText asText ;
		yourself! !

!MauiPage methodsFor: 'access' stamp: 'cmm 6/5/2012 22:05'!
embeddedMauiMorphs
self maMarked: 'delete'.
	^ text 
		ifNil: [ Array empty ]
		ifNotNil: [ text embeddedMorphs select: [ : each | each isMauiMorph ] ]! !

!MauiPage methodsFor: 'access' stamp: 'cmm 4/1/2008 23:12'!
embeddedMorphs
	^ text embeddedMorphs! !

!MauiPage methodsFor: 'access' stamp: 'cmm 6/18/2011 15:07'!
maContextKeywordsDo: aBlock
	self subStrings do: aBlock! !

!MauiPage methodsFor: 'access' stamp: 'cmm 6/18/2011 15:09'!
string
	^ text ifNil: [ String empty ] ifNotNil: [ text string ]! !

!MauiPage methodsFor: 'access' stamp: 'cmm 11/19/2007 21:17'!
subStrings
	^ self string subStrings! !

!MauiPage methodsFor: 'access' stamp: 'cmm 11/19/2007 20:56'!
text
	^ text! !

!MauiPage methodsFor: 'initialize' stamp: 'cmm 10/26/2007 13:33'!
text: aText
	text := aText! !

!MauiPage methodsFor: 'access' stamp: 'cmm 12/20/2007 23:18'!
textStream
	^ TextStream with: self text! !

!MauiPage methodsFor: 'access' stamp: 'cmm 12/21/2007 00:21'!
textStreamDo: oneArgBlock 
	"Value oneArgBlock with a textStream that will then replace my 'text' contents."
	| modifiedTextStream |
	modifiedTextStream := oneArgBlock value: self textStream.
	self text: modifiedTextStream contents! !

!MauiPage methodsFor: 'copying' stamp: 'cmm 10/30/2007 12:15'!
veryDeepInner: aDeepCopier
	super veryDeepInner: aDeepCopier.
	text := text veryDeepCopyWith: aDeepCopier! !

!Object class methodsFor: '*ma-core' stamp: 'cmm 4/21/2009 18:13'!
registerSelfAsFileReader
	(FileServices registeredFileReaderClasses includes: self) ifFalse: [ FileServices registerFileReader: self ]! !

!Object methodsFor: '*ma-core' stamp: 'cmm 12/30/2007 18:41'!
copyAttributes: anArray from: sourceObject

	anArray do:
		[ :each |
		self
			instVarNamed: each asString
			put: (sourceObject instVarNamed: each) copy ]! !

!Object methodsFor: '*ma-core' stamp: 'cmm 10/9/2012 20:29'!
differencesWith: anObject 
	^ self
		differencesWith: anObject
		using: #=! !

!Object methodsFor: '*ma-core' stamp: 'cmm 11/26/2012 21:03'!
differencesWith: anObject using: comparisonOperator 
	| differences c1 c2 |
	differences := Dictionary new.
	self == anObject ifTrue: [ ^ Dictionary new ].
	c1 := self maOriginalClass.
	c2 := anObject maOriginalClass.
	c1 instSpec = c2 instSpec ifFalse:
		[ differences
			at: 'class'
			put:
				(Array
					with: self
					with: anObject) ].
	(c1 isBytes and: [ self ~= anObject ]) ifTrue:
		[ differences at:
			(Array
				with: 'byte-content'
				with: self
				with: anObject) ].
	(c1 isWords and:
		[ c1 isPointers not and: [ self ~= anObject ] ]) ifTrue:
		[ differences
			at: 'word-content'
			put:
				(Array
					with: self
					with: anObject) ].
	(c1 allInstVarNames intersection: c2 allInstVarNames) do:
		[ : each | | my his | ((my := (self instVarNamed: each) yourself)
			perform: comparisonOperator
			with: (his := (anObject instVarNamed: each) yourself)) ifFalse:
			[ differences
				at: each
				put:
					(Array
						with: my
						with: his) ] ].
	(c1 allInstVarNames difference: c2 allInstVarNames) do:
		[ : each | differences
			at: each
			put:
				(Array
					with: (self instVarNamed: each) yourself
					with: 'no inst var named ' , each) ].
	(c2 allInstVarNames difference: c1 allInstVarNames) do:
		[ : each | differences
			at: each
			put:
				(Array
					with: 'no inst var named ' , each
					with: (anObject instVarNamed: each) yourself) ].
	1
		to: (self basicSize max: anObject basicSize)
		do:
			[ : index | index > self basicSize
				ifTrue:
					[ differences
						at: index
						put:
							(Array
								with: 'no reference at index ' , index asString
								with: (anObject basicAt: index) yourself) ]
				ifFalse:
					[ index > anObject basicSize
						ifTrue:
							[ differences
								at: index
								put:
									(Array
										with: (anObject basicAt: index) yourself
										with: 'no reference at index ' , index asString) ]
						ifFalse:
							[ ((self basicAt: index) yourself
								perform: comparisonOperator
								with: (anObject basicAt: index) yourself) ifFalse:
								[ differences
									at: index
									put:
										(Array
											with: (self basicAt: index) yourself
											with: (anObject basicAt: index) yourself) ] ] ] ].
	^ differences! !

!Object methodsFor: '*ma-core' stamp: 'cmm 10/9/2012 18:20'!
identityDifferencesWith: anObject 
	^ self
		differencesWith: anObject
		using: #==! !

!Object methodsFor: '*ma-core' stamp: 'cmm 5/15/2011 12:50'!
isEmptyOrNil
	^ false! !

!Object methodsFor: '*ma-core' stamp: 'cmm 11/15/2009 15:24'!
isInMagma
	^ (self respondsTo: #magmaSession) and: [ self magmaSession notNil ]! !

!Object methodsFor: '*ma-core' stamp: 'cmm 8/21/2004 15:53'!
maAbbreviatedString
	"This is intended to support two possible string representations for any object
	instead of just one.  The normal one and this abbreivated one.  The normal one provides type-information, the abbreviation provides the identifier.  An abbreviated
	provides a brevity that proves useful when looking at collections or
	other 'container' type objects.

	Suggested usage is to omit the objects type (e.g. class name) for printing abbreviated,
	but include it for standard printOn:"

	^ String streamContents: [ : stream | self maPrintAbbreviatedOn: stream ]! !

!Object methodsFor: '*ma-core' stamp: 'cmm 2/15/2002 00:02'!
maIsException

	^false
! !

!Object methodsFor: '*ma-core' stamp: 'cmm 7/15/2006 15:12'!
maIsSortedCollection
	^ false! !

!Object methodsFor: '*ma-core' stamp: 'cmm 11/10/2005 22:07'!
maMarked: aString
	"This is a marker method used to indicate that we wish to come back and re-examine
	code in the future for possible improvement.  For now, try to keep improvements categorized
	via anEsAtom that we can Browse References to, if necessary.  The possibilities should be:

		'ugly'
		'performance'
		'bug'
		'dev'
		'vmSpecific'
		'releaseSpecific'
		'otherPackageSupport'
		'system override'
		'deprecated'
		'magic'
		'debug'
	A *comment* then should follow to provide more detail."

	aString = 'delete' ifTrue: [ self halt: 'Oops..  looks like you still need this method.' ]! !

!Object methodsFor: '*ma-core' stamp: 'cmm 2/7/2008 16:56'!
maOriginalClass
	"Unfortunately, we don't have complete class-transparency with WriteBarrier.  #species isn't always the quite the thing needed.  In rare cases (browse my senders), my actual class is needed."
	^ self class! !

!Object methodsFor: '*ma-core' stamp: 'cm 12/29/2001 01:14'!
maPrintAbbreviatedOn: aStream
	"This is intended to support two possible string representations for any object
	instead of just one.  The normal one and this abbreivated one.  An abbreviated
	provides a brevity that proves useful when looking at collections or
	other 'container' type objects.

	Suggested usage is to omit the objects type (e.g. class name) for printing abbreviated,
	but include it for standard printOn:"
! !

!Object methodsFor: '*ma-core' stamp: ''!
maPrintAttribute: aSymbol on: aStream

	(self class allInstVarNames includes: aSymbol asString)
		ifTrue:
			[ self
				maPrintInstVar: aSymbol
				on: aStream ]
		ifFalse:
			[ self
				maPrintSelector: aSymbol
				on: aStream ]
! !

!Object methodsFor: '*ma-core' stamp: ''!
maPrintAttributes: arrayOfSymbols on: aStream

	arrayOfSymbols do:
		[ :each |
		self
			maPrintAttribute: each
			on: aStream ]
! !

!Object methodsFor: '*ma-core' stamp: 'cmm 12/19/2002 23:26'!
maPrintInstVar: aSymbol on: aStream

	| value |

	aStream
		cr;
		tab;
		nextPutAll: aSymbol;
		nextPutAll: ' = '.
	value := self instVarNamed: aSymbol asString.
	(value isCollection and: [ value isString not ])
		ifTrue:
			[ value do:
				[ :each |
				aStream
					maPrint: each;
					space ] ]
		ifFalse:
			[ aStream maPrint: value ]
! !

!Object methodsFor: '*ma-core' stamp: 'cmm 7/30/2007 00:23'!
maPrintOn: aStream
	self printOn: aStream! !

!Object methodsFor: '*ma-core' stamp: 'cmm 5/2/2008 11:48'!
maPrintSelector: aSymbol on: aStream

	| value |

	aStream
		cr;
		tab;
		nextPutAll: aSymbol;
		nextPutAll: ' : '.
	value :=
		[ self perform: aSymbol ]
			on: Error
			do: [ :exc | exc return: (exc messageText ifNil: [ 'error' ]) ].
	(value isCollection and: [ value isString not ])
		ifTrue:
			[ value do:
				[ :each |
				aStream
					maPrint: each;
					space ] ]
		ifFalse:
			[ aStream maPrint: value ]! !

!Object methodsFor: '*ma-core' stamp: 'cmm 7/24/2002 20:46'!
maShallowCopy

	^self shallowCopy! !

!Object methodsFor: '*ma-core' stamp: 'cmm 6/17/2010 15:28'!
maSuggestedFilename
	"Answer a name that would be suitable, if the receiver were to be saved in a file."
	^ self name asFileName ! !

!Object methodsFor: '*ma-core' stamp: 'cmm 7/9/2008 20:39'!
realObject
	"Balance API with MagmaForwardingProxy, so applications may sometimes choose to get hold of the actual object, not just a forwardingProxy."
	^ self! !

!MCRepository methodsFor: '*ma-core' stamp: 'cmm 3/18/2009 21:10'!
isDirectoryRepository
	^ false! !

Warning subclass: #MaWarning
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Core-Exceptions'!

!MaWarning methodsFor: 'as yet unclassified' stamp: 'cmm 9/16/2008 15:35'!
defaultAction
	SmalltalkImage current maIsHeadless
		ifTrue: [ self resume ]
		ifFalse: [ super defaultAction ]! !
MaSingleton initialize!
MaCoreConstants initialize!
